#-*-:coding:utf-8-*-

def read(urlFile):
    return "\{begin}..."

def extractContent(completeDocument):
    """ Function to separate the document from the preamble(LaTeX formating)
            and the information after the document. It returns an array of strings
            with the parts.

            Args:
                    completeDocument: The first parameter.

            Returns:
                    array: It returns an array of strings with the parts.

    """
    #preamble = ""
    #document = ""
    #epilogue = ""

    try:
        preamble = completeDocument[:completeDocument.index(r'\begin{document}')]
        document = completeDocument[completeDocument.index(r'\begin{document}'): completeDocument.index(r'\end{document}')+ len(r'\end{document}')]
        epilogue = completeDocument[completeDocument.index(r'\end{document}')+ len(r'\end{document}'):]
    except ValueError:
        print("\\begin{document} or \end{document} not found.\n")
    return [preamble, document, epilogue]
#EndOfFunction



"""A LaTeX to TXT file translator."""
import click
from .main import translate

__version__ = "0.1.1"

@click.command()
@click.argument('file_name')
@click.version_option(__version__)
def cli(file_name):
	translate(file_name)
	click.echo("Traducido correctamente")



#-*-:coding:utf-8-*-
import re
from . import translator as tl

def translate(file_name):
	codigo = tl.string(file_name, False)
	#Revisa si hay archivos incrustados en el tex principal para agregarlos a codigo
	patron = re.compile(r'\\input *\{(?P<archivo>[^\{\}#]*)\}')
	ArchivosHijos = patron.findall(codigo)
	patron=re.compile(r'\.tex$')
	for i in ArchivosHijos:
		if len(patron.findall(i))==0: j=i+'.tex'
		else: j=i
		codigo = codigo.replace('\\input{'+i+'}', tl.string(j, True))
	codigo = tl.graficos(codigo)
	formulas=tl.buscar_formulas(codigo, 1)
	ecuaciones=tl.buscar_formulas(codigo, 2)
	codigos=tl.buscar_codigos(codigo)
	codigo=tl.reemplazar_formulas(codigo, ecuaciones, 'ecuacion', 'r')
	codigo=tl.reemplazar_formulas(codigo, formulas, 'formula', 'r')
	codigo=tl.reemplazar_formulas(codigo, codigos, 'codigo', 'r')
	ecuaciones=tl.traducir_formulas(ecuaciones)
	formulas=tl.traducir_formulas(formulas)
	codigo = tl.titulos(codigo)
	codigo = tl.limpiar_basura(codigo)
	codigo = tl.enumeracion(codigo)
	codigo=tl.reemplazar_formulas(codigo, ecuaciones, 'ecuacion', 'w')
	codigo=tl.reemplazar_formulas(codigo, formulas, 'formula', 'w')
	codigo=tl.reemplazar_formulas(codigo, codigos, 'codigo', 'w')
	codigo=tl.split_codigo(codigo)
	codigo=re.sub(r'\n{3,}', r'\n\n', codigo)
	codigo=codigo.replace('\n', '\r\n')
	tl.traducido(codigo, f"{file_name[:-4]} traducido.txt")
	return f"{file_name[:-4]} traducido.txt"



# -*-:coding:utf-8-*-

import os
import copy
import json
import string
import subprocess
#from blindtex import mainBlindtex
import sys
from sys import argv
import codecs
# HU1
# Method to open a file and return its content as a string.


def openFile(fileName):
    '''This function takes a file a return its content as a string.
            Args:
                    fileName(str): The name of the file to be oppened.
            Returns:
                    str: The content of the file.'''
    try:
        myFile = codecs.open(fileName, mode='r', errors='ignore')
        stringDocument = myFile.read()
        myFile.close()
        return stringDocument
    except IOError:
        print("File %s could not be openned." % (fileName))
        return ""
# EndOfFunction


def read_json_file(fileName):
    '''This function takes a file a return its content as a string.
            Args:
                    fileName(str): The name of the file to be oppened.
            Returns:
                    str: The content of the file.'''
    try:
        with codecs.open(fileName, encoding='utf-8', mode='r', errors='ignore') as myFile:
            stringDocument = json.load(myFile)

    except OSError as err:
        print("OS error: {0}".format(err))
        raise
    except ValueError:
        print("Could not parser", fileName, "file, please check json syntax.")
        raise
    except:
        print("Unexpected error:", sys.exc_info()[0])
        raise

    return stringDocument


# Replace the document containing the LaTeX math with the output of the function seekAndReplace. Write the content in a new file.
def replaceAndWrite(contentList, replacedDocument, fileName):
    '''Replace the document containing the LaTeX math with the output of the function seekAndReplace. Write the content in a new file.
            Args:
                    contentList(list[str,str,str]): The list generated by extractContent.
                    replacedDocument(str): the LaTeX content without formulas, just markers.
                    fileName(str): The name of the .tex file where the result will be written. '''
    newContentList = copy.deepcopy(contentList)
    newContentList[1] = replacedDocument
    try:
        # TODO Check if the file already exits, warn about that and decide if the user wants to replace it.
        myFile = open(fileName, 'w')
        myFile.write(string.join(newContentList))
        myFile.close()
    except IOError:
        print("File could not be oppened.")
        return


# EndOfFunction

def convertToHtml(fileName, biblioName=None):
    '''This function uses LaTeXML to convert a .tex file in a html with accesible math formulas.
            Args:
                    fileName(str): the name of the .tex file to be processed.
                    (opt)biblioName(str): the name o a .bib file. '''

    noExtensionName = fileName.replace(".tex", "")

    if(biblioName):
        if(os.name == 'nt'):  # i.e is in windows
            noExtensionBiblio = biblioName.replace(".bib", "")
            subprocess.call(["latexml", "--dest=%s.xml" %
                            (noExtensionName), "--quiet", fileName], shell=True)
            subprocess.call(["latexml", "--dest=%s.xml" %
                            (noExtensionBiblio), "--bibtex", biblioName], shell=True)
            subprocess.call(["latexmlpost", "-dest=%s.xhtml" % (noExtensionName),
                            "--bibliography=%s.xml" % (noExtensionBiblio), noExtensionName+".xml"], shell=True)
        else:  # TODO: Do not repeat
            noExtensionBiblio = biblioName.replace(".bib", "")
            subprocess.call(["latexml", "--dest=%s.xml" %
                            (noExtensionName), "--quiet", fileName])
            subprocess.call(["latexml", "--dest=%s.xml" %
                            (noExtensionBiblio), "--bibtex", biblioName])
            subprocess.call(["latexmlpost", "-dest=%s.xhtml" % (noExtensionName),
                            "--bibliography=%s.xml" % (noExtensionBiblio), noExtensionName+".xml"])
    else:
        if(os.name == 'nt'):
            # Generates xml file.
            subprocess.call(["latexml", "--dest=%s.xml" %
                            (noExtensionName), "--quiet", fileName], shell=True)
            # Generates xhtml file.
            subprocess.call(["latexmlpost", "-dest=%s.xhtml" %
                            (noExtensionName), noExtensionName+".xml"], shell=True)
        else:
            # Generates xml file.
            subprocess.call(["latexml", "--dest=%s.xml" %
                            (noExtensionName), "--quiet", fileName])
            # Generates xhtml file.
            subprocess.call(["latexmlpost", "-dest=%s.xhtml" %
                            (noExtensionName), noExtensionName+".xml"])


# EndOfFunction

def convertToPdf(filePath, fileName):
    if(os.name == 'nt'):
        subprocess.call(['pdflatex', '-output-directory',
                        filePath, fileName], shell=True)
        subprocess.call(['pdflatex', '-output-directory',
                        filePath, fileName], shell=True)
    else:
        subprocess.call(['pdflatex', '-output-directory', filePath, fileName])
        subprocess.call(['pdflatex', '-output-directory', filePath, fileName])


# EndOfFunction

# TODO ¿con alguna extensión o la extensión se da desde afuera?
def writeHtmlFile(htmlString, fileName):
    '''Function to write the html result in a final file.
            Args:
                    htmlString(str): The string with the html content of the final result.
                    fileName(str): The name of the file where the string will be written. '''
    try:
        htmlFile = open(fileName, 'w')
        htmlFile.write(htmlString)
        htmlFile.close()
    except IOError:
        print('File could not be oppened.')
        return
# EndOf Function


# This function works just when a .tex file is being converted.
def writeTroubles(strfileName, listtroubleFormulas):
    (filePath, name) = os.path.split(strfileName)
    try:
        registerFile = open(os.path.join(
            filePath, 'TroubleFormulasOf'+name.replace('.tex', '.txt')), 'w')
        for formula in listtroubleFormulas:
            registerFile.write('I had troubles with:\n'+formula+'\n')
        registerFile.close()
    except IOError:
        return
# EndOfFunction



from latextranslator.translator import traducir_formulas

def test_equation_1():
    formulas = ["$\\lim_{x \\to 0} \\frac{1}{x}$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["lim_(x→0)(1/x)"]

def test_equation_2():
    formulas = ["$f'(a)  \\sim \\frac{f(x) - f(a)}{x-a}$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["f'(a)∼(f(x)−f(a))/(x−a)"]

def test_equation_3():
    formulas = ["$f'(a)  \\sim \\frac{f(x) - f(a)}{x-a}$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["f'(a)∼(f(x)−f(a))/(x−a)"]

def test_equation_4():
    formulas = ["$E=mc^2$", "$a^2 + b^2 = c^2$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["E=mc^2", "a^2+b^2=c^2"]

def test_equation_5():
    formulas = ["$\\int_{0}^{1} x^2 dx$", "$\\sum_{n=1}^{5} \\frac{1}{n}$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["∫[0, 1] x^2dx", "∑[n=1, 5] (1/n)"]

def test_equation_6():
    formulas = ["$\\lim_{x\\to\\infty} \\frac{1}{x}=0$", "$\\sqrt{9}=3$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["lim_(x→∞)(1/x)=0", "√9=3"]

def test_equation_7():
    formulas = ["$E=mc^2$", "$x^2 + y^2 = z^2$", "$\\int_{0}^{1} x^2 dx$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["E=mc^2", "x^2+y^2=z^2", "∫[0, 1] x^2dx"]

def test_equation_8():
    formulas = ["$\\Sigma_{i=1}^{n} a_i$", "$\\int_{a}^{b} f(x) dx$", "$\\sqrt{2}$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["Σ^n_(i=1)a_i", "∫[a, b] f(x)dx", "√2"]

def test_equation_9():
    formulas = ["$\\lim_{x\\to\\infty} f(x)$", "$\\nabla\\cdot\\vec{F}$", "$A\\cup B$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["lim_(x→∞)f(x)", "∇·F⃗", "A∪B"]

def test_equation_10():
    formulas = ["$\\forall x, \\exists y : P(x, y)$", "$\\neg (A \\land B)$", "$\\mathcal{L}(x)$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["∀x,∃y: P(x,y)", "¬(A∧B)", "L(x)"]

def test_equation_11():
    formulas = ["$\\psi(\\theta) = A\\sin(\\theta) + B\\cos(\\theta)$", "$f''(x) + f(x) = 0$", "$\\partial F/\\partial t$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["ψ(θ)=Asin(θ)+Bcos(θ)", "f''(x)+f(x)=0", "∂F÷∂t"]

def test_equation_12():
    formulas = ["$\\Phi(x) \\leftrightarrow \\exists y : Q(y)$", "$\\alpha, \\beta, \\gamma$", "$\\Delta x = \\frac{b-a}{n}$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["Φ(x)flechaIzquierdaDerecha∃y: Q(y)", "α,β,γ", "∆x=(b−a)/(n)"]

def test_equation_13():
    formulas = ["$x_i^{(n)}$", "$P(X=x)$", "$A\\subset B$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["x^((n))_i", "P(X=x)", "A⊂B"]

def test_equation_14():
    formulas = ["$\\sum_{n=1}^{\\infty} \\lim_{{x\\to\\infty}} f_n(x)$"]
    resultado = traducir_formulas(formulas)
    assert resultado == ["∑[n=1, ∞] lim_(x→∞)f_n(x)"]
    


# -*-:coding:utf-8-*-
# Diccionario que contiene todoas las palabras ordinarias que se pueden utilizar
# Funciones: en esta sección, dejaremos todas las funciones que se requieran
from pkg_resources import _sget_dict
import json
import os

class dictionary(object):
    '''Dictionary class containing LaTeX commands and its possible readings.
    Attributes:
        dict(dictionary): Is the principal dictionary, its items are of the form "command(str) : [conf(int), readings(list of strings)]"
                        command is a LaTeX command, conf(self.dict[command][0]) is the chosen reading at the moment, acts as a index,
                         readings(self.dict[command][1]) is a list of all the possible readings of command.
        fileName(str): Is the name of the json file where the dictionary will be saved.'''    
    def __init__(self, *args, **kwargs):
        '''Args:
                *args: jsonFileName(str): Is the name of the file from where the dictionary will be charged.
                **kwargs: dict = myDictionary(dict) : a dictionary to work, it will be not saved until saveAs command is called.'''
        if ('dict' in kwargs):
            self.dict = kwargs['dict']
        else:
            self.open(args[0])

    #--------------------------------------------------------------------------

    def open(self, jsonFileName):
        '''Function to open and charge the dictionary from a json file.
            Args:
                jsonFileName(str): Is the name of the file from where the dictionary will be charged. '''
        self.fileName = jsonFileName                
        try:
            myFile = open(jsonFileName,'r')
            self.dict = json.load(myFile)
            myFile.close()
        except IOError:
            print('File %s could not be oppened.'%jsonFileName)
    #--------------------------------------------------------------------------
    def save(self):
        '''Function to save in fileName the dictionary dict.'''
        try:
            myFile = open(self.fileName, 'w')
            json.dump(self.dict, myFile)
            myFile.close()
        except IOError:
            print('File could not be oppened.')
    #--------------------------------------------------------------------------
    def saveAs(self,newJsonFileName):
        '''Function to create a new json file to save the dictionary.
            Args:
                newJsonFileName(str): The name of the file where the dictionary will be saved.'''
        self.fileName = newJsonFileName + '.json'
        try:
            myFile = open(newJsonFileName + '.json', 'w')
            json.dump(self.dict, myFile)
            myFile.close()
        except IOError:
            print('File could not be oppened.')
    #--------------------------------------------------------------------------
    def addReading(self, key, newValue):
        '''Function to add a reading to a existing key.
            Args:
                key(str): The command that will have a new reading.
                newValue(str): The new reading to the command.'''
        try:
            self.dict[key][1].append(newValue)
            #self.dict[key][0] = len(self.dict[key][1]) - 1
        except KeyError:
            print('Command %s does not exist.'%key)
    #--------------------------------------------------------------------------
    def changeReadingIndex(self, key, index):
        '''Function to change the reading of a command.
        Args:
            key(str): The command that will change its reading.
            index(int): The index of the new reading.''' 
        try:
            if(index < len(self.dict[key][1])):
                self.dict[key][0] = index                
            else:
                print('The value number %d is not in the readings list.'%index)
        except KeyError:
            print('Command %s does not exist.'%key)
    #--------------------------------------------------------------------------
    #TODO Agrega un comando con su respectiva lectura
    def addCommand(self,key,value):#Agregar el comando y la lectura Puede variar cuando las configuraciones estén listas.
        if (self.dict.get(str(key)) is not None):
            return False
        else:
            try:
                #Add the command to the dicctionary in regexes, to be recognized.
                #Open the regexes file.
                myFile = open(os.path.join('converter','dicts','regexes.json'), 'r')#THIS IS NOT THE PATH
                dictOfDicts = json.load(myFile)
                myFile.close()
                #Adds the new command in the dict of Dicts
                (path, dictName) = os.path.split(self.fileName)
                dictName  = dictName.replace('.json','')
                dictOfDicts[dictName] = dictOfDicts[dictName] + "|%s"%key
                myFile = open(os.path.join('converter','dicts','regexes.json'), 'w')
                json.dump(dictOfDicts, myFile)
                myFile.close()
            except IOError:
                print('File %s could not be oppened.'%'regexes.json')
                return False
            #Add the command with its reading to the dictionary itself.
            self.dict[(key)] = [0,[(value)]]
        return True
    #--------------------------------------------------------------------------

    def changeReading(self,key,value):#Agregar una lectura nueva
        '''Function to change the reading of a command.
        Args:
            key(str): The command that will change its reading.
            value(str): The new reading, if it exists.'''
        try:
            self.dict[(key)][0] = self.dict[(key)][1].index((value))
        except KeyError:
            print('Command %s does not exist.'%key)
        except IndexError:
            print('Reading %s does not exist.'%value)
    #--------------------------------------------------------------------------

    def showReading(self,key):#Retorna la lectura configurada, falta implementar para lectura de más de una configuración
        '''Function to show the current reading a command has.
            Args:
                key(str): The command to be read.
            Returns:
                str: The reading pointed by conf in the list.'''
        try:
            #dict(key)[0] is the index of the reading (the configuration) in the list that is dict(key)[1].
            # So, we are accessing the value in the index dict(key)[0] that is in the list dict(key)[1]. 
            return (self.dict.get(key)[1][self.dict.get(key)[0]])
        except KeyError:
            #print('Command %s does not exist.'%key)
            return key #This is because I think if the dictionary does not have the command it is better to return the same command than crash the program. David-Casas
    #---------------------------------------------------------------------------
    def showReadingIndex(self,key, index):
        try:
            return (self.dict.get(key)[1][index])
        except KeyError:
            #print('Command %s does not exist.'%key)
            return key #See ShowReading
    #--------------------------------------------------------------------------
    
    def showReadings(self, key):
        '''Function to show all the readings that a command has.
            Args:
                key(str): The command.'''
        try:
            return(self.dict[key][1])
        except KeyError:
            print('Command %s does not exist.'%key)

    #--------------------------------------------------------------------------            
    def showLatex(self,value):#Devuelve el comando al que está asociado el valor
        '''Function to sohw the command of a reading, it such exists.
            Args:
                value(str): The reading the user wants to know tha associated command.
            Returns:
                str: The command of the reading. '''
        theValueExists = False
        for item in self.dict.items():
            if value in item[1][1]:
                key = item[0]
                theValueExists = True
                break
        if(theValueExists):
            return key
        else:
            return 'The value %s has no LaTeX command associated.'%value
    #--------------------------------------------------------------------------

    def isThere(self,key):
        '''
            Function to know if certain command is in the dictionary.
            Args:
                key(str): The possible command.
            Returns:
                bool: True if the command is in the dict, false otherwise. 
        '''
        if (key in self.dict):
            return (True)
        else:
            return (False)



#-*-:coding:utf-8-*-
import blindtex.iotools.iotools
import blindtex.iotools.stringtools
from latextranslator.blindtex.iotools.stringtools import troubleFormulas
from latextranslator.blindtex.converter import parser
import argparse
import os
import os.path
from sys import argv
import sys
reload(sys)
sys.setdefaultencoding('utf-8')

def convertDocument(fileName):

    #Get the document in a string.
    documentString = iotools.iotools.openFile(fileName)
    #Get the content in a list with three elements.
    documentContent = iotools.stringtools.extractContent(documentString)
    #This generates a list(a named tuple) with the document content (all formulas replaced) named replacedDocument, inline formulas named inlineList and display formulas named displayList.
    documentAndLists = iotools.stringtools.seekAndReplaceFormulas(documentContent[1])
    newDocumentContent = documentAndLists.replacedDocument
    #Generate the labels list
    labelsList = iotools.stringtools.generateListOfLabels(documentAndLists.displayList)
    #Replace the references.
    newDocumentContent = iotools.stringtools.replaceRefs(newDocumentContent, labelsList)
    #Let's deal with path and fileNames, actually fileName is the path of the file.
    (filePath,name) = os.path.split(fileName)
    #Write another tex file without formulas.
    iotools.iotools.replaceAndWrite(documentContent,newDocumentContent,os.path.join(filePath,'noFormula_'+name))
    iotools.iotools.convertToHtml(os.path.join(filePath,'noFormula_'+name))

    #converter.parser.setOption(0)
    #Convert the formulas
    for index in range(len(documentAndLists.inlineList)):
        documentAndLists.inlineList[index] = converter.parser.convert(documentAndLists.inlineList[index])

    for index in range(len(documentAndLists.displayList)):
        documentAndLists.displayList[index] = converter.parser.convert(documentAndLists.displayList[index])

    #Get the html in a string
    htmlString = iotools.iotools.openFile(os.path.join(filePath,'noFormula_'+name.replace('.tex','.xhtml')))
    #Insert converted formulas.
    htmlString = iotools.stringtools.insertConvertedFormulas(htmlString, documentAndLists.inlineList, documentAndLists.displayList)
    #Insert References
    htmlString = iotools.stringtools.insertReferences(htmlString,labelsList)
    iotools.iotools.writeHtmlFile(htmlString, os.path.join(filePath,name.replace('.tex','.xhtml')))

    #Remove Residues
    os.remove(os.path.join(filePath,'noFormula_'+name))
    os.remove(os.path.join(filePath,'noFormula_'+name.replace('.tex','.xml')))
    os.remove(os.path.join(filePath,'noFormula_'+name.replace('.tex','.xhtml')))

    #Write the trouble formulas
    iotools.iotools.writeTroubles(fileName, troubleFormulas)

#EndOfFunction

def convertToPdf(fileName):
    #Get the document in a string.
    documentString = iotools.iotools.openFile(fileName)
    #Get the content in a list with three elements.
    documentContent = iotools.stringtools.extractContent(documentString)
    #This generates a list(a named tuple) with the document content (all formulas replaced) named replacedDocument, inline formulas named inlineList and display formulas named displayList.
    documentAndLists = iotools.stringtools.seekAndReplaceFormulas(documentContent[1])
    newDocumentContent = documentAndLists.replacedDocument
    #Generate the labels list
    labelsList = iotools.stringtools.generateListOfLabels(documentAndLists.displayList)

    #Let's deal with path and fileNames, actually fileName is the path of the file.
    (filePath,name) = os.path.split(fileName)

    converter.parser.setOption(3)#For the LaTeX Accents
    #Convert the formulas
    for index in range(len(documentAndLists.inlineList)):
        #Here we take the risk of the sign ~ being used in the formula.
        documentAndLists.inlineList[index] = "\# %s \#"%converter.parser.convert(documentAndLists.inlineList[index])

    for index in range(len(documentAndLists.displayList)):
        documentAndLists.displayList[index] = "Ecuaci\\'on\\\\%s\\\\ Fin Ecuaci\\'on\\\\"%converter.parser.convert(documentAndLists.displayList[index])
    #insert converted formulas
    newDocumentContent = iotools.stringtools.insertConvertedFormulas( newDocumentContent, documentAndLists.inlineList, documentAndLists.displayList)

    #Write the .tex file with the modified formulas
    iotools.iotools.replaceAndWrite(documentContent,newDocumentContent,os.path.join(filePath,'Accessible_'+name))

    iotools.iotools.convertToPdf(filePath ,os.path.join(filePath,'Accessible_'+name))

    #Write the trouble formulas
    iotools.iotools.writeTroubles(fileName, troubleFormulas)
#EndOfFunction
def convertFormula(strFormula,intOption = 1):

    converter.parser.setOption(intOption)
    return converter.parser.convert(strFormula)
#EndOfFunction


if __name__=='__main__':
    parser = argparse.ArgumentParser(description="Flip a switch by setting a flag")

    parser.add_argument('-e','--equation', dest='equation',
                                            help = 'Latex format equation to convert',
                                            default="")

    parser.add_argument('-o','--output', dest='document',
                                            help = '',
                                            default="")


    args = parser.parse_args()

    if args.equation:
        print("Equation: ", converter.parser.convert(args.equation))
        print("Equation: ", args.equation)
    else:
        print('The name is %s'% args.document)
        convertDocument(args.document)
#EndOfMain



#-*-:coding:utf-8-*-
class MathObject():

    def __init__(self,
                 content = None,
                 children = None,
                 left_superscript = None,
                 left_subscript = None,
                 accent = None,
                 above = None,
                 under = None,
                 superscript = None,
                 subscript = None,
                 left_delimiter = None,
                 right_delimiter = None,
                 style = None,
                 kind = None,):

        self.children = children #These are the children of the node in a list.
        self.content = content

        #Attributes of Math Object proposed by Raman
        self.left_superscript = left_superscript
        self.left_subscript = left_subscript
        self.accent = accent
        self.above = above
        self.under = under
        self.superscript = superscript
        self.subscript = subscript
        self.left_delimiter = left_delimiter
        self.right_delimiter = right_delimiter
        self.style = style
        self.kind = kind
        #The kind is the use of the math_object Ord, binary operator, arrow...etc. Just the symbols will have it.
        #This will serve to know beforehand in what dictionary look for and if it has some special behaviour.
        #It is not convenient to put kind in block, fraction, root...etc for the further exploration of the formula.

    def append_child(self, node_new_child):
        if(self.children == None):
            self.children = node_new_child
        else:
            self.children.append(node_new_child)

    #TODO: Verify if the list has de capacity for the index and so.
    def add_child(self, index, node_new_child):
        if(self.children == None):
            self.children = node_new_child
        else:
            self.children.insert(index, node_new_child)

    def get_children(self):
        return self.children
#EndOfClass



from latextranslator.blindtex.latex2ast import lexer
from latextranslator.blindtex.latex2ast import parser

def latex2list(latex_srt):
    par = parser.get_parser()
    lex = lexer.get_lexer()

    return par.parse(latex_srt,lex)



def translate_lineal_read(list_of_tokens, dictionary):
    """
    Args:
        list_of_tokens(list): List of token to translate
        dictionary(dict): Python dictionary with tokens and their traduction

    Returns
        dict: List of token translated using dictinary
    """
    list_translated = []
    for token in list_of_tokens:
        if dictionary.get(token):
            list_translated.append(dictionary[token])
        else:
            list_translated.append(token)
    return list_translated



#-*-:coding:utf-8-*-

import ply.lex
import ply.yacc
from latextranslator.blindtex.latex2ast import lexer
from latextranslator.blindtex.latex2ast.math_object import MathObject

tokens = lexer.tokens

precedence = (
        ('right', 'SUP', 'SUB'),
)

# In this parser the end result will be a python list with math_objects as elements.
# Each math object will have some characteristics.
def p_start(p):
    """
    start : formula
    """
    p[0] = p[1]
#If formula is nothing, the list does not exist, so it is oppened; if not,
# to the list we add the new math object.
def p_formula(p):
    """
    formula : math_object
            | concat
    """
    if(p[0] == None):
        p[0] = p[1]
    else:
        p[0] = p[0] + p[1]

def p_lnbrk(p):
    '''lnbrk : LINEBREAK '''
    p[0] = MathObject(content = r'\\')

def p_col(p):
    '''col : COL '''
    p[0] = MathObject(content = '&')
    
#Rule to deal with concatenation of math objects.
def p_formulas(p):
    '''concat :  math_object formula'''
    if(p[0] == None):
        p[0] = p[1] + p[2]
    else:
        p[0] = p[0] + p[1] +p[2]

def p_math_object(p):
    '''math_object : symbol
                            | block
            | fraction
            | root
            | choose
            | binom
            | pmod
            | text
            | label
            | array
            | lnbrk
            | col'''
    p[0] = [p[1]]#It returns a list of one object.

def p_block(p):
    """
    block : BEGINBLOCK formula ENDBLOCK
    """
    p[0] = MathObject(content = 'block')
    p[0].append_child(p[2])

def p_accent(p):
    '''
    math_object : ACCENT block
                    | ACCENT symbol
    '''
    p[2].accent = p[1]
    p[0] = [p[2]]#Are you sure this does not create a conflict if the MathObject is complex?

def p_overset(p):
    '''math_object : OVERSET argument argument '''
    p[3].above = p[2]
    p[0] = [p[3]]#Are you sure this does not create a conflict if the MathObject is complex?

def p_underset(p):
    '''math_object : UNDERSET argument argument '''
    p[3].under = p[2]
    p[0] = [p[3]]#Are you sure this does not create a conflict if the MathObject is complex?

def p_style(p):
    '''
    math_object : STYLE block
                    | STYLE symbol
    '''
    p[2].style = p[1]
    p[0] = [p[2]]#Are you sure this does not create a conflict if the MathObject is complex?

def p_formula_scripted(p):
    '''
    math_object : math_object simple_scripted
                    | math_object compound_scripted
    '''
    p[0] = p[1]

#All the possible things an argument could be.
def p_argument(p):
    '''argument : symbol
                            | block'''
    p[0] = [p[1]]

#All the possible things a scripty may be.
def p_script(p):
    '''script : symbol
                    | block
                    | fraction
                    | root
                    | binom
                    | pmod
                    | text'''
    p[0] = [p[1]]

def p_simple_scripted(p):
    '''
    simple_scripted : SUP script
                                    | SUB script
    '''
    if(p[1] == '^'):
        #This distintion is made for formulas like a^2 ^3 (double scripted)
        #LaTeX accepts them and we have to.
        #Here we create a new Node with content 'nothing_scripted' that will be processed later. 
        if(p[-1][0].superscript == None):
            p[-1][0].superscript = p[2]#Adds the script to the previous element.
        else:
            p[-1].append(MathObject(content = 'nothing_scripted', superscript = p[2]))#What a duct tape fix.
    else:
        if(p[-1][0].subscript == None):
            p[-1][0].subscript = p[2]
        else:
            p[-1].append(MathObject(content = 'nothing_scripted', subscript = p[2]))

def p_compound_scripted(p):
    '''
    compound_scripted : SUP script SUB script
                                    | SUB script SUP script
    '''
    if(p[1] == '^'):
        #See p_simple_scripted
        if(p[-1][0].superscript == None and p[-1][0].subscript == None):
            #If some is not and the parser reached here, something funny is happening.
            p[-1][0].superscript = p[2]
            p[-1][0].subscript = p[4]
        else:
            p[-1].append(Node(content = 'nothing_scripted', superscript = p[2], subscript = p[4]))
    else:
        if(p[-1][0].superscript == None and p[-1][0].subscript == None):
            p[-1][0].subscript = p[2]
            p[-1][0].superscript = p[4]
        else:
            p[-1].append(Node(content = 'nothing_scripted', superscript = p[4], subscript = p[2]))

def p_symbol(p):
    """
    symbol : LIM
            | UNKNOWN
            | MOD
            | "!"
            | NOT
                        | KNOT
                        | USER
    """
    p[0] = MathObject(content = p[1])

def p_ord(p):
    ''' symbol :  ORD '''
    p[0] = MathObject(content = p[1], kind = 'Ordinary')#Beware to distinguish if is in the dictionary or not.

def p_noKind_ord(p):
    '''symbol : NUM
               | CHAR '''
    p[0] = MathObject(content = p[1])

def p_largeOp(p):
    ''' symbol : LARGEOP '''
    p[0] = MathObject(content = p[1], kind = 'LargeOperators')

def p_binop(p):
    ''' symbol : BINOP
                       | KBINOP '''
    p[0] = MathObject(content = p[1], kind = 'BinaryOperators')#Beware to distinguish if is in the dictionary or not.

def p_binrel(p):
    ''' symbol : KBINREL
                            | BINREL'''
    p[0] = MathObject(content = p[1], kind = 'BinaryRelations')#Beware to distinguish if is in the dictionary or not.

def p_func(p):
    ''' symbol : FUNC '''
    p[0] = MathObject(content = p[1], kind = 'MathFunctions')

def p_arrow(p):
    ''' symbol : ARROW '''
    p[0] = MathObject(content = p[1], kind = 'Arrows')

def p_delimiter(p):
    ''' symbol : DELIMITER
                            | kdelimiter '''
    p[0] = MathObject(content = p[1], kind = 'Delimiters')

def p_dots(p):
    ''' symbol : DOTS '''
    p[0] = MathObject(content = p[1], kind = 'Dots')

#We count [ and ] apart because the \sqrt structure uses them.
def p_kdelimiter(p):
    '''kdelimiter : KDELIMITER
                    | "["
                    | "]" '''
    p[0] = p[1]

def p_fraction(p):
    '''fraction : FRAC argument argument '''
    p[0] = MathObject(content = 'fraction')
    p[0].append_child(p[2])
    p[0].append_child(p[3][0]) #This is because p[3] is a list, as index returns that, but we want to append the MathObject, not the list.

def p_root(p):
    '''root : sqr_root
            | indexed_root '''
    p[0] = p[1]

def p_sqr_root(p):
    '''sqr_root : ROOT argument '''
    p[0] = MathObject(content = 'root')
    p[0].append_child(p[2])

def p_indexed_root(p):
    '''indexed_root : ROOT root_index argument '''
    p[0] = MathObject(content = 'root')
    p[0].append_child(p[2])#The first child will be the index.
    p[0].append_child(p[3][0])#This is because p[5] is a list, as index returns that, but we want to append the MathObject, not the list.

#TODO:Here argument must be changed by formula, but this change generates lots of problems.
def p_root_index(p):
    'root_index : "[" argument "]" '
    p[0] = p[2]

def p_choose(p):
    ''' choose : formula CHOOSE formula '''
    p[0] = MathObject(content = 'choose')
    p[0].append_child(p[1])
    p[0].append_child(p[3][0])

#Something funny here. See test_binom
def p_binom(p):
    '''binom : BINOM argument argument '''
    p[0] = MathObject(content = 'binom')
    p[0].append_child(p[2])
    p[0].append_child(p[3][0])

def p_pmod(p):
    ''' pmod : PMOD argument '''
    p[0] = MathObject(content = 'pmod')
    p[0].append_child(p[2])

def p_text(p):
    ''' text : TEXT TCHAR
            | TEXT ANYTHING'''
    p[0] = MathObject(content = 'text')
    p[0].append_child(p[2])

def p_label(p):
    '''label : LABEL '''
    p[0] = MathObject(content = 'label')
    p[0].append_child(p[1])

def p_array(p):
    '''array : BEGARRAY start ENDARRAY '''
    p[0] = MathObject(content = 'array')
    p[0].append_child(p[2])

#Here we have a delay of the problem. If  LINEBREAK is included as the
# row separator there is a conflict with LINEBREAK as symbol in any 
# formula, so, I decided to postpone the row and column finding in the later processes.
#By the moment it creates a large list with all the rows in one.


def get_parser():
    return ply.yacc.yacc()

if __name__ == "__main__":
    parser = get_parser()
    lexer = lexer.get_lexer()
    #cv = parser.parse(latex_string,custom_lexer)#,debug=1)
    while True:
        try:
            try:
                s = raw_input()
            except NameError: # Python3
                s = input('spi> ')

            cv_s = parser.parse(s,lexer)
            print(cv_s)
        except EOFError:
            break



#-*-:coding:utf-8-*-

import json
import ply.lex as lex
from ply.lex import TOKEN
import json
import re

tokens = ('CHAR', 'SUP', 'SUB',
          'BEGINBLOCK','ENDBLOCK','ORD',
          'FRAC', 'ROOT', 'LARGEOP',
          'BINOP','KBINOP','KBINREL',
          'BINREL', 'NOT', 'FUNC',
          'ARROW','KDELIMITER','DELIMITER',
          'ACCENT','STYLE','DOTS',
          'LIM','UNKNOWN','BEGARRAY',
          'ENDARRAY','LINEBREAK','COL',
          'CHOOSE','BINOM','PMOD',
          'PHANTOM','TEXT','LABEL',
          'ANYTHING','USER',
          'NUM','KNOT', 'MOD', 'TCHAR','UNDERSET', 'OVERSET')

dictOfDicts = json.loads("{\"Dots\": \"dots|ldots|cdots|vdots|ddots\", \
    \"Styles\": \"mathit|mathrm|mathbf|mathsf|mathtt|mathcal|boldmath\", \
    \"LargeOperators\": \"sum|prod|coprod|int|oint|bigcap|bigcup|bigsqcup|bigvee|bigwedge|bigodot|bigotimes|bigoplus|biguplus|limits\", \"Delimiters\": \"{|}|lfloor|rfloor|lceil|rceil|langle|rangle|backslash|vert\", \
    \"Arrows\": \"leftarrow|Leftarrow|rightarrow|Rightarrow|leftrightarrow|Leftrightarrow|mapsto|hookleftarrow|leftharpoonup|leftharpoondown|rightleftharpoons|longleftarrow|Longleftarrow|longrightarrow|Longrightarrow|longleftrightarrow|Longleftrightarrow|longmapsto|hookrightarrow|rightharpoonup|rightharpoondown|uparrow|Uparrow|downarrow|Downarrow|updownarrow|Updownarrow|nearrow|searrow|swarrow|nwarrow|to|gets|iff\", \
    \"Accents\": \"hat|check|breve|acute|grave|tilde|bar|vec|dot|ddot|widehat|widetilde\", \"MathFunctions\": \"arccos|arcsin|arctan|arg|cos|cosh|cot|coth|csc|deg|det|dim|exp|gcd|hom|inf|ker|lg|liminf|limsup|ln|log|max|min|Pr|sec|sin|sinh|sup|tan|tanh\", \
    \"BinaryOperators\": \"pm|mp|setminus|cdot|times|ast|star|diamond|circ|bullet|div|cap|cup|uplus|sqcap|sqcup|triangleleft|triangleright|wr|bigcirc|bigtriangleup|bigtriangledown|vee|wedge|oplus|ominus|otimes|oslash|odot|dagger|ddagger|amalg|lor|land|bmod\", \
    \"UserDict\": \"SafetyPorpouses\", \
    \"Ordinary\": \"(alpha)|(beta)|gamma|delta|epsilon|varepsilon|zeta|eta|theta|vartheta|iota|kappa|lambda|mu|nu|xi|pi|varpi|rho|varrho|sigma|varsigma|tau|upsilon|phi|varphi|chi|psi|omega|Gamma|Delta|Theta|Lambda|Xi|Pi|Sigma|Upsilon|Phi|Psi|Omega|aleph|hbar|imath|jmath|ell|vp|Re|Im|partial|infty|prime|emptyset|nabla|surd|top|bot|\\\\|angle|triangle|backslash|forall|exists|neg|flat|natural|sharp|clubsuit|diamondsuit|heartsuit|spadsuit|lnot|prime\", \
    \"BinaryRelations\": \"leq|geq|equiv|prec|succ|sim|preceq|succeq|simeq|ll|gg|asymp|subset|supset|approx|subseteq|supseteq|cong|sqsubseteq|sqsupseteq|bowtie|in|ni|propto|vdash|dashv|models|smile|mid|doteq|frown|parallel|perp|neq|notin|ne|(le)|ge|owns\"}")

def get_lexer():

    states = (('command', 'exclusive'),#)
              ('anything','exclusive'),
               ('text', 'exclusive'),)

    literals = [ '!',"'", '[', ']' ]

    BEGINBLOCK = r'\{'
    ENDBLOCK = r'\}'
    SUP = r'\^'
    SUB = r'_'
    COMMAND = r'\\'
    command_PMOD = r'pmod'
    command_MOD = r'mod|bmod'
    command_PHANTOM = r'[hv]?phantom'
    command_BEGARRAY = r'(begin\{array\}|begin\{[pbBvV]?matrix(\*)?\})(\[.*?\])?(\{.*?\})?'
    command_ENDARRAY = r'end\{array\}|end\{[pbBvV]?matrix(\*)?\}'
    command_LINEBREAK = r'\\'
    command_UNDERSET = r'underset'
    command_OVERSET = r'overset'
    COL = r'[&]'
    #command_LARGEOP =
    #command_ORD =
    command_FRAC = r'frac|tfrac|dfrac'
    command_ROOT = r'sqrt'
    #command_ARROW =
    command_leftRight = r'(left)|(right)|left\.|right\.'
    KBINOP = r'\+|-|\*|/'
    #command_DOTS =
    #command_BINOP =
    KBINREL = r'[=<>]'
    #command_BINREL =
    #command_FUNC =
    KNOT = r'¬'
    command_NOT = r'not' #Negate binary relations
    KDELIMITER = r'\(|\)'
    #command_DELIMITER =
    #command_ACCENT =
    #command_STYLE =
    command_LIM = r'lim'
    command_CHOOSE = r'choose'
    command_BINOM = r'binom'
    command_MATHSPACE = r'[,!:;]|quad|qquad'
    command_TEXT = r'(text(rm)?|mbox)'
    text_TCHAR = r'[A-Za-z0-9"¡!¿?,;.:()¬]'#TODO: Make this more inclusive.
    text_ignore_SPACE = r'[ \t\n]+'
    text_TBBLOCK = r'\{'
    command_LABEL = r'label\{(.)*(?<!\\)\}'
    #ARRAYTEXT = r'~text\{'
    anything_ANYTHING = r'[^}]+'
    anything_ENDANY = r'(?<!\\)\}'
    CHAR = r'[A-Za-z"%\',.:;|]+?'
    #NUM = r'[0-9]?'
    NUM = r'[0-9]{1,}' # Extraer numeros de 1 o mas digitos
    #command_USER =
    command_UNKNOWN = r'[A-Za-z]+'
    t_ignore_SPACE=r'[ \t\n]+' #LaTeX does not care about spaces in math mode unless they are specified.

    #dictOfDicts = json.loads("{\"Dots\": \"dots|ldots|cdots|vdots|ddots\", \
    #\"Styles\": \"mathit|mathrm|mathbf|mathsf|mathtt|mathcal|boldmath\", \
    #\"LargeOperators\": \"sum|prod|coprod|int|oint|bigcap|bigcup|bigsqcup|bigvee|bigwedge|bigodot|bigotimes|bigoplus|biguplus|limits\", \"Delimiters\": \"{|}|lfloor|rfloor|lceil|rceil|langle|rangle|backslash|vert\", \
    #\"Arrows\": \"leftarrow|Leftarrow|rightarrow|Rightarrow|leftrightarrow|Leftrightarrow|mapsto|hookleftarrow|leftharpoonup|leftharpoondown|rightleftharpoons|longleftarrow|Longleftarrow|longrightarrow|Longrightarrow|longleftrightarrow|Longleftrightarrow|longmapsto|hookrightarrow|rightharpoonup|rightharpoondown|uparrow|Uparrow|downarrow|Downarrow|updownarrow|Updownarrow|nearrow|searrow|swarrow|nwarrow|to|gets|iff\", \
    #\"Accents\": \"hat|check|breve|acute|grave|tilde|bar|vec|dot|ddot|widehat|widetilde\", \"MathFunctions\": \"arccos|arcsin|arctan|arg|cos|cosh|cot|coth|csc|deg|det|dim|exp|gcd|hom|inf|ker|lg|liminf|limsup|ln|log|max|min|Pr|sec|sin|sinh|sup|tan|tanh\", \
    #\"BinaryOperators\": \"pm|mp|setminus|cdot|times|ast|star|diamond|circ|bullet|div|cap|cup|uplus|sqcap|sqcup|triangleleft|triangleright|wr|bigcirc|bigtriangleup|bigtriangledown|vee|wedge|oplus|ominus|otimes|oslash|odot|dagger|ddagger|amalg|lor|land|bmod\", \
    #\"UserDict\": \"SafetyPorpouses\", \
    #\"Ordinary\": \"(alpha)|(beta)|gamma|delta|epsilon|varepsilon|zeta|eta|theta|vartheta|iota|kappa|lambda|mu|nu|xi|pi|varpi|rho|varrho|sigma|varsigma|tau|upsilon|phi|varphi|chi|psi|omega|Gamma|Delta|Theta|Lambda|Xi|Pi|Sigma|Upsilon|Phi|Psi|Omega|aleph|hbar|imath|jmath|ell|vp|Re|Im|partial|infty|prime|emptyset|nabla|surd|top|bot|\\\\|angle|triangle|backslash|forall|exists|neg|flat|natural|sharp|clubsuit|diamondsuit|heartsuit|spadsuit|lnot|prime\", \
    #\"BinaryRelations\": \"leq|geq|equiv|prec|succ|sim|preceq|succeq|simeq|ll|gg|asymp|subset|supset|approx|subseteq|supseteq|cong|sqsubseteq|sqsupseteq|bowtie|in|ni|propto|vdash|dashv|models|smile|mid|doteq|frown|parallel|perp|neq|notin|ne|(le)|ge|owns\"}" )

    @TOKEN(BEGINBLOCK)
    def t_BEGINBLOCK(t):
        return t

    @TOKEN(ENDBLOCK)
    def t_ENDBLOCK(t):
        return t

    @TOKEN(SUP)
    def t_SUP(t):
        return t

    @TOKEN(SUB)
    def t_SUB(t):
        return t

    @TOKEN(COMMAND)
    def t_COMMAND(t):
        t.lexer.begin('command')
        return

    @TOKEN(command_MOD)
    def t_command_MOD(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_PMOD)
    def t_command_PMOD(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_PHANTOM)
    def t_command_PHANTOM(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_BEGARRAY)
    def t_command_BEGARRAY(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_ENDARRAY)
    def t_command_ENDARRAY(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_LINEBREAK)
    def t_command_LINEBREAK(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_UNDERSET)
    def t_command_UNDERSET(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_OVERSET)
    def t_command_OVERSET(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(COL)
    def t_COL(t):
        return t

    @TOKEN(dictOfDicts['LargeOperators'])
    def t_command_LARGEOP(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['Ordinary'])
    def t_command_ORD(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_FRAC)
    def t_command_FRAC(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_ROOT)
    def t_command_ROOT(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['Arrows'])
    def t_command_ARROW(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_leftRight)
    def t_command_leftRight(t):
        t.lexer.begin('INITIAL')
        pass

    @TOKEN(KBINOP)
    def t_KBINOP(t):#Binary operators that can be made from the keyboard.
        return t

    @TOKEN(dictOfDicts['Dots'])
    def t_command_DOTS(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['BinaryOperators'])
    def t_command_BINOP(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(KBINREL)
    def t_KBINREL(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['BinaryRelations'])
    def t_command_BINREL(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['MathFunctions'])
    def t_command_FUNC(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_NOT)
    def t_command_NOT(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(KNOT)
    def t_KNOT(t):
        return t

    @TOKEN(KDELIMITER)
    def t_KDELIMITER(t):
        return t

    @TOKEN(dictOfDicts['Delimiters'])
    def t_command_DELIMITER(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['Accents'])
    def t_command_ACCENT(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(dictOfDicts['Styles'])
    def t_command_STYLE(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_LIM)
    def t_command_LIM(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_CHOOSE)
    def t_command_CHOOSE(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_BINOM)
    def t_command_BINOM(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_MATHSPACE)
    def t_command_MATHSPACE(t):
        t.lexer.begin('INITIAL')
        pass

    @TOKEN(command_TEXT)
    def t_command_TEXT(t):
        t.lexer.begin('text')
        return t

    @TOKEN(text_TCHAR)
    def t_text_TCHAR(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(text_TBBLOCK)
    def t_text_TBBLOCK(t):
        t.lexer.begin('anything')
        pass

    @TOKEN(text_ignore_SPACE)
    def t_text_ignore_SPACE(t):
        pass

    @TOKEN(command_LABEL)
    def t_command_LABEL(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(anything_ANYTHING)
    def t_anything_ANYTHING(t):
        return t

    @TOKEN(anything_ENDANY)
    def t_anything_ENDANY(t):
        t.lexer.begin('INITIAL')
        pass

    @TOKEN(CHAR)
    def t_CHAR(t):
        return t

    @TOKEN(NUM)
    def t_NUM(t):
        return t

    @TOKEN(dictOfDicts['UserDict'])
    def t_command_USER(t):
        t.lexer.begin('INITIAL')
        return t

    @TOKEN(command_UNKNOWN)
    def t_command_UNKNOWN(t):
        t.lexer.begin('INITIAL')
        return t

    def t_command_error(t):
        print("Illegal character '%s'" % t.value[0])
        #t.lexer.skip(1)
        #raise illegalCharacter

    def t_error(t):
        print("Illegal character '%s'" % t.value[0])
        #t.lexer.skip(1)
        #raise illegalCharacter

    def t_anything_error(t):
        print("Illegal character '%s'" % t.value[0])

    def t_text_error(t):
        print("Illegal character '%s'" % t.value[0])

    return lex.lex(reflags = re.UNICODE)

if __name__ =="__main__":
    lexer = get_lexer()
    while True:
        try:
            try:
                s = raw_input()
            except NameError: # Python3
                s = input('spi> ')
        except EOFError:
            break

        lexer.input(s)
        while True:
            tok = lexer.token()
            if not tok:
                break

            print(tok)



#-*-:coding:utf-8-*-
from latextranslator.blindtex.latex2ast import converter
#from latex2ast import ast
from latextranslator.blindtex.interpreter import dictionary

to_read = {'simple_superscript' : '^ %s ',
                'comp_superscript' : '^( %s ) ',
                'simple_subscript' : '_ %s ',
                'comp_subscript' : '_( %s ) ',
                'simple_frac' : ' ( %s over %s ) ',
                'comp_frac' : 'fraction %s ) over ( %s endFraction ',
                'simple_sqrt' : 'squareRootOf ( %s ) ',
                'comp_sqrt' : 'squareRootOf ( %s ) ',
                'simple_root' : 'sqrt %s ( %s ) ',
                'comp_root' : 'sqrt %s ( %s ) ',
                'simple_choose' : 'from %s choose %s ',
                'comp_choose' : 'from %s choose %s endChoose ',
                'simple_modulo' : 'modulo %s ',
                'comp_modulo' : 'modulo %s endModulo ',
                'simple_text' : 'text %s ',
                'comp_text' : 'text %s endText ',
                'from_to' : '[ %s , blindtexespacioblindtex %s ] blindtexespacioblindtex ',
                'over' : '[ %s ] blindtexespacioblindtex ',
                'to' : 'to %s of ',
                'end' : 'end%s ',
                'linebreak' : 'linebreak',
                'array' : 'array %s endArray ',
                'array_element' : 'element',}
'''dict: A dictionary with the strings to read some mathematical structures,
    like: Fractions, roots, large operators, etc.'''
#Function to convert a math_object in a string.
def lineal_read(Node):
    '''Function to convert a mathObject into a string.
    Args:
        Node(mathObject): A mathObject created by parser.
        
    Returns: 
    String: A string with the lineal read of the mathObject.'''
    #Dictionary with the possible nodes with children and the functions each case call.
    with_children = {'block' : lineal_read_block,
                     'fraction' : lineal_read_fraction,
                     'root' : lineal_read_root,
                     'choose' : lineal_read_choose_binom,
                     'binom' : lineal_read_choose_binom,
                     'pmod' : lineal_read_pmod,
                     'text' : lineal_read_text,
                     'label' : lineal_read_label,
                     'array' : lineal_read_array}

    str_lineal_read = ''

    #The attributes will be readed in this order:
    # accent -> content-> children* -> style -> superscript -> subscript
    #TODO: Add the option for the user to change this.
    #TODO: Add the option to ommit style or whatever.
    #TODO: Add the translation by dictionary.



    #I'll go for the easiest and less elegant way: A chain of ifs.
    
    if(Node.content in with_children): #If the node has children.
        #Identify the type of children the node has and act accordingly
        str_lineal_read = str_lineal_read + with_children[Node.content](Node.children)
    else:
        str_lineal_read = str_lineal_read + lineal_read_content(Node)

    if(Node.style != None):#Add the style of the node.
        str_lineal_read = lineal_read_style(Node, str_lineal_read)

    if(Node.accent != None): #The accent
        str_lineal_read = lineal_read_accent(Node, str_lineal_read)

    #This part is to read the limits of large Operators like integral or sum.
    if(Node.kind == 'LargeOperators'):
        if(Node.subscript != None and Node.superscript != None):
            str_lineal_read = str_lineal_read + to_read['from_to']%(lineal_read(Node.subscript[0]),
                                                                    lineal_read(Node.superscript[0]))
        elif(Node.subscript != None and Node.superscript == None):
            str_lineal_read = str_lineal_read + to_read['over']%lineal_read(Node.subscript[0])

        elif(Node.subscript == None and Node.superscript != None):
            str_lineal_read = str_lineal_read + to_read['to']%lineal_read(Node.superscript[0])
    else:#If the math_object is not a LargeOperator but has scripts.
        if(Node.superscript != None):
            str_lineal_read = str_lineal_read + lineal_read_superscript(Node.superscript[0])

        if(Node.subscript != None):
            str_lineal_read = str_lineal_read + lineal_read_subscript(Node.subscript[0])

    return str_lineal_read
#EndOfFunction

#Function to read the content of the math_object
def lineal_read_content(node):
    '''A function to returns as a string the content of a mathObject.
    Args:
    node(mathObject)
    Returns: 
    string: A string with the content of the mathObject. '''
    if(node.kind != None):
        return '%s '%node.content
    elif(node.content == r'\\'):
        return to_read['linebreak']
    else:
        return '%s '%node.content
#EOF
#To read the accent
def lineal_read_accent(node, str_read):
    '''
    Args: 
        node(mathObject)
        str_read(string): String with the reading so far.
    Returns:
        string: The string with the proper accent of the mathObject in the linear read. '''
    
    if(is_simple(node)):
        return str_read + '%s '%node.accent
    else:
        return '%s '%node.accent + str_read + to_read['end']%node.accent
#EOF
#To read the style.
def lineal_read_style(node, str_read):
    '''
    Args: 
        node(mathObject)
        str_read(string): String with the reading so far.
    Returns:
        string: The string with the proper stylr of the mathObject in the linear read. '''
    if(is_simple(node)):
        return str_read
    else:
        return str_read
#EOF

def lineal_read_superscript(node_script):
    '''
    
    Args:
        node_script(mathObject): the mathObject that is a super script of another mathObject.
    Returns:
        string: The reading of this object that changes wether it is simple or not.
    '''
    #Here if the script is a block it will be considered as a compound script.
    #This to avoid the following ambiguity: a^{b_c} and a^b_c
    #That, otherwise, would be read the same.
    if(is_simple(node_script) and node_script.content != 'block'):
        return to_read['simple_superscript']%lineal_read(node_script)
    else:
        return to_read['comp_superscript']%lineal_read(node_script)
#EndOfFunction

def lineal_read_subscript(node_script):
    '''
    
    Args:
        node_script(mathObject): the mathObject that is a sub script of another mathObject.
    Returns:
        string: The reading of this object that changes wether it is simple or not.
    '''
    #See lineal_read_superscript.
    if(is_simple(node_script) and node_script.content != 'block'):
        return to_read['simple_subscript']%lineal_read(node_script)
    else:
        return to_read['comp_subscript']%lineal_read(node_script)
#EndOfFunction


def is_simple(Node):
    '''A node is simple if it is just a character e.g. "a", "2", "\alpha"
        or if it is a block with just one character in it e.g. "{a}", "{2}", "{\alpha}"
    Args:
        node(mathObject)
    Returns:
        bool: True if the node is simple.
    '''
    if(Node.get_children() == None):
        return True
    elif(Node.content == 'block' and len(Node.get_children()) == 1 ):
        return(is_simple(Node.get_children()[0]))#This is for cases like {{a+b}} (not simple) or {{{\alpha}}}(simple).
    else:
        return False
#EndOfFunction

def lineal_read_block(list_children):
    '''A Block has from one to several children, thus functions reads those.
    Args:
        list_childre(list of mathObject): The list of a block children, they al are mathObject.
    Returns:
        string:The linear read of such children.
    '''
    # The child of a block is a formula, a formula is a list of nodes.
    str_result = ''
    for node in list_children:
        str_result = str_result + lineal_read(node)

    return str_result
#EndOfFunction

def lineal_read_fraction(list_children):
    '''A fraction is simple if both of its parts are simple, otherwise the function will tell where it begins 
    and where it ends.
    Args:
        list_children(list of mathObjects): A list of lenght 2, the first is the numeratos the second the denominator.
    Returns:
        string: The reading of the fraction depending wether it is simple or not.'''
    #The list received here must be of lenght 2. The numerator and denominator.
    if(is_simple(list_children[0]) and is_simple(list_children[1])):
        return to_read['simple_frac']%(lineal_read(list_children[0]), lineal_read(list_children[1]))
    else:
        return to_read['comp_frac']%(lineal_read(list_children[0]), lineal_read(list_children[1]))
#EndOfFunction

def lineal_read_root(list_children):
    '''Here we have four cases: Square root simple or not, other root simple or not.
        In the not simple roots it tells where it begins and where it ends.
    Args:
        list_children(list of mathObjects): The arguments of a root, could be
            one if the root is square, of two if the root has other index, in this case
            the first mathObject is the index and the second the object of the root.
    Returns:
        string: The reading of the root depending wether it is simple or not.
    '''
    #There are two cases here: Either the root has an index \sqrt[i]{k} or not.
    if(len(list_children) == 1):#It has not index
        if(is_simple(list_children[0])):
            return to_read['simple_sqrt']%(lineal_read(list_children[0]))
        else:
            return to_read['comp_sqrt']%(lineal_read(list_children[0]))
    else:
        if(is_simple(list_children[1])):
            return to_read['simple_root']%(lineal_read(list_children[0]), lineal_read(list_children[1]))
        else:
            return to_read['comp_root']%(lineal_read(list_children[0]), lineal_read(list_children[1]))
#EndOfFunction

def lineal_read_choose_binom(list_children):
    '''A binomial coefficient is simple if both of its parts are simple, otherwise the function will tell where it begins 
    and where it ends.
    Args:
        list_childre(list of mathObjects): There are two arguments of a binomial
        coeffiecient.
    Returns:
        string: The reading of the binomial coefficient.
    '''
    if(is_simple(list_children[0]) and is_simple(list_children[1])):
        return to_read['simple_choose']%(lineal_read(list_children[0]), lineal_read(list_children[1]))
    else:
        return to_read['comp_choose']%(lineal_read(list_children[0]), lineal_read(list_children[1]))
#EndOfFunction

def lineal_read_pmod(list_children):
    '''If the argument of the module is not simple the function will
    tell where the module begins and where it ends.
    
    Args:
        list_children(list of mathObjects): A single mathObject.
    Returns:
        string: The reading of the module depending wether is simple or not.
    '''
    if(is_simple(list_children[0])):
        return to_read['simple_modulo']%(lineal_read(list_children[0]))
    else:
        return to_read['comp_modulo']%(lineal_read(list_children[0]))
#EndOfFunction

def lineal_read_text(list_children):
    '''
    Args:
        list_children(list of mathObjects): The mathObject with the internal text.
    Returns:
        string: The reading of the text. It indicates where it begins and
        where it ends.
    '''
    return 'text %s endtext'%(list_children)#Here the child is a string
#EndOfFunction

def lineal_read_label(list_children):
    '''As labels are important of LaTeX self reference every label is left
    the same.
    Args:
        list_children(list of mathObjects): The mathObject with the label.
    Returns:
        string: The same label.
    '''
    return r'\%s'%(list_children)#The labels must be inaltered.
#EndOFFunction

def lineal_read_formula(list_formula):
    '''The parser of blindtex takes a formula in LaTeX and divide it into
    the different mathObjects present in the formula, this list of mathObjects
    is passed to this function.
    Args:
        list_formula(list of mathObjects): The list of all the mathObjects
        that shape the whole formula.
    Returns:
        string: The total lineal read of the formula, shaped by the lineal
        reads of every mathObject.
    '''
    str_result = ''
    for node in list_formula:
        str_result = str_result + lineal_read(node)
    return str_result
#EndOfFunction



def lineal_read_array(list_array):
    '''
    Args:
        list_array(list of mathObjects): The elemenst of the array and
        the delimitators of such array (strings like '&' and '\\')
    Returns:
        string: the lineal read of the array, distinguishing each element by a
        title that indicates its position (first its row and then its column)
        e.g. "element 1_2 a", indicates that the element is in the first row
        and second column. Also the result indicates where the array beings and
        where it ends.
    '''
    nrow = 1
    ncol = 1
    str_result = '%s%d_%d '%(to_read['array_element'],nrow, ncol)
    array = list_array

    for element in array:
        str_element_reading = lineal_read(element)
        if(str_element_reading == '& '):
            ncol += 1
            str_result += '%s%d_%d '%(to_read['array_element'],nrow, ncol)
            continue
        elif(str_element_reading == to_read['linebreak']):
            nrow += 1
            ncol = 1
            str_result += '%s%d_%d '%(to_read['array_element'],nrow, ncol)
            continue
        str_result += '%s '%str_element_reading

    return to_read['array']%str_result
#EndOfFunction

def lineal_read_formula_list(list_formula):
    str_result = ''
    for node in list_formula:
        str_result = str_result + lineal_read(node)
    return str_result.split()
#EndOfFunction

if __name__ == "__main__":
    while True:
        try:
            try:
                s = raw_input()
            except NameError: # Python3
                s = input('spi> ')

            cv_s = converter.latex2list(s)
            print(lineal_read_formula(cv_s))
        except EOFError:
            break



import argparse
import os
import pkg_resources
from latextranslator.blindtex.latex2ast import converter
from latextranslator.blindtex.interpreter import reader
from latextranslator.blindtex.lang import translator
from latextranslator.blindtex.iotools import iotools

DEFAULT_DICT_PATH = os.path.join("lang", "dicts", "spanish.json")


def read_equation(latex_equation, mode='lineal', dictionary_path=''):

    # User enter custom dictionary path ?
    if dictionary_path == '':
        # Read of blindtex dictionary
        dictionary = iotools.read_json_file(
            pkg_resources.resource_filename(__package__, DEFAULT_DICT_PATH))
    else:
        # Read of custom file
        dictionary = iotools.read_json_file(dictionary_path)

    if mode == 'lineal':
        list_object_math = converter.latex2list(latex_equation)
        list_lineal_reading = reader.lineal_read_formula_list(list_object_math)
        list_lineal_reading_translated_to_spanish = translator.translate_lineal_read(
            list_lineal_reading, dictionary)
    return ' '.join(list_lineal_reading_translated_to_spanish)


def main():
    parser = argparse.ArgumentParser(
        description="Tool for LaTeX's equations convertion")

    parser.add_argument('-e', '--equation', dest='equation',
                        help='Latex equation to translate',
                        default='')

    parser.add_argument('-d', '--dictionary', dest='dictionary_path',
                        help='Path to dictinary in json format',
                        default='')
    args = parser.parse_args()

    if args.equation:
        print(read_equation(args.equation, 'lineal', args.dictionary_path))
    else:
        parser.print_help()



translations={
	'simbolos': (
		('\\\\', 'blindtexlineablindtex', '\n'),
		('\\ ', 'blindtexespacioblindtex', ' '),
		('\\:', 'blindtexespacioblindtex', ' '),
		('\\;', 'blindtexespacioblindtex', ' '),
		('\\,', 'blindtexespacioblindtex', ' '),
		('\\quad', 'blindtexespacioblindtex', ' '),
		('\\qquad', 'blindtexespacioblindtex'*2, ' '),
		('\\%', 'blindtexporcientoblindtex', '%'),
		('\\|', 'blindtexbarraverticalblindtexblindtexbarraverticalblindtex', '||'),
		('|', 'blindtexbarraverticalblindtex', '|'),
		('#', 'blindtexsignonumeroblindtex', '#'),
		('°', 'blindtexgradosblindtex', '°'),
		('\\hdots', 'blindtexhdotsblindtex', '⋯'),
		('\\therefore', 'blindtexthereforeblindtex', '∴'),
		('\\varnothing', 'blindtexvarnothingblindtex', '∅'),
		('\\subseteq', 'blindtexsubseteqblindtex', '⊆'),
		('\\veebar', 'blindtexveebarblindtex', '⊻'),
		('\\nexists', 'blindtexnexistsblindtex', '∄'),
		('\\implies', 'blindteximpliesblindtex', '⇒'),
		('~', 'blindtextildeblindtex', '~'),
		('?', 'blindtexcierrainterrogacionblindtex', '?'),
		('¿', 'blindtexabreinterrogacionblindtex', '¿'),
		('á', 'blindtexatildeblindtex', 'á'),
		('Á', 'blindtexamayusculatildeblindtex', 'Á'),
		('é', 'blindtexetildeblindtex', 'é'),
		('É', 'blindtexemayusculatildeblindtex', 'É'),
		('í', 'blindtexitildeblindtex', 'í'),
		('Í', 'blindteximayusculatildeblindtex', 'Í'),
		('ó', 'blindtexotildeblindtex', 'ó'),
		('Ó', 'blindtexomayusculatildeblindtex', 'Ó'),
		('ú', 'blindtexutildeblindtex', 'ú'),
		('Ú', 'blindtexumayusculatildeblindtex', 'Ú')
	),
	'reemplazos_latex': (
		('\\[', '['),
		('−', '-'),
		('\\lbrack', '['),
		('\\lbrace', '\\{'),
		('\\rbrace', '\\}'),
		('\\rbrack', ']'),
		('\\|', '\\Vert'),
		('\\lvert', '|'),
		('\\Vert', '||'),
		('\\vert', '|'),
		('\\rvert', '|'),
		('\\colon', ':'),
		('\\mapsto', '\\rightarrow'),
		('\\overrightarrow', '\\vec')
	),
	'reemplazos_traduccion': (
		("blindtexpesoblindtex", "$"),
		("-", "−"),
		("blindtexvectorblindtex", "⃗"),
		("^(')", "'"),
		('( ', "("),
		(' )', ")"),
		('{ ', "{"),
		(' }', "}")
	),
	'basura_latex': (
		'\\ ',
		'_{}',
		'^{}',
		'\\pause',
		'\\medskip',
		'\\bigskip',
		'\\smallskip',
		'\\displaystyle',
		'\\limits',
		'\\nolimits',
		'\\rm',
		'\\rm\n',
		'\\Bigg',
		'\\bigg',
		'\\Big',
		'\\mathbb',
		'\\mathrm',
		'\\mathcal',
		'\\big',
		'\n'
	)
}


from latextranslator.translator import titulos

codigo = """

Este es el contenido de la diapositiva 1.
\\frametitle{Título de la diapositiva 2}
Este es el contenido de la diapositiva 2.
"""

def test_titulos_1():
    assert titulos("\\frametitle{Título de la diapositiva 1}") == "Título de la diapositiva 1\n"

def test_titulos_2():
    assert titulos("\\frametitle{\\frametitle\\frametitle}") == "\\frametitle\\frametitle\n"

# Caso de prueba para '\frametitle'
def test_frametitle():
    codigo = "\\frametitle{Título de la diapositiva}\nEste es el contenido de la diapositiva."
    resultado = titulos(codigo)
    assert resultado == "Título de la diapositiva\n\nEste es el contenido de la diapositiva."

# Caso de prueba para '\framesubtitle'
def test_framesubtitle():
    codigo = "\\framesubtitle{Subtítulo de la diapositiva}\nEste es el contenido de la diapositiva."
    resultado = titulos(codigo)
    assert resultado == "Subtítulo de la diapositiva\n\nEste es el contenido de la diapositiva."

# Caso de prueba para '\begin{block}'
def test_begin_block():
    codigo = "\\begin{block}{Título del bloque}\nContenido del bloque.\n\\end{block}"
    resultado = titulos(codigo)
    assert resultado == "Título del bloque\n\nContenido del bloque.\n\\end{block}"

# Caso de prueba para '\begin{frame}'
def test_begin_frame():
    codigo = "\\begin{frame}{Título del cuadro}\nContenido del cuadro.\n\\end{frame}"
    resultado = titulos(codigo)
    assert resultado == "\n\nContenido del cuadro.\n\\end{frame}"

# Caso de prueba para múltiples ocurrencias
def test_multiple_ocurrencias():
    codigo = "\\frametitle{Título 1}\nContenido 1.\n\\frametitle{Título 2}\nContenido 2."
    resultado = titulos(codigo)
    assert resultado == "Título 1\n\nContenido 1.\nTítulo 2\n\nContenido 2."

# Caso de prueba para un código sin patrones
def test_sin_patrones():
    codigo = "Este es un código sin patrones de título."
    resultado = titulos(codigo)
    assert resultado == codigo



#-*-:coding:utf-8-*-

import re
import string
import collections
import copy
#import blindtex.converter.parser

#Regular expression to match anything in math mode. Altough there are better regex to do the same, this allows add new options easily.
#If you want to proove it before,  you can do it in https://regex101.com/r/dSxw4f/2/
#TODO The repetition of code is a signal it can be made in a more elegant way.

inlineMath = re.compile(r'''(?<!\\)( #Exclude escaped symbols
                                                                ((\$)(.*?)(?<!\\)(\$))| #Single $ formulas
                                                                ((\\\()(.*?)(?<!\\)(\\\))) #\(
                                                        )''', re.DOTALL|re.UNICODE|re.X)
displayMath = re.compile(r'''(?<!\\)( #Exclude escaped symbols
                                                                                ((\${2})(.*?)(?<!\\)(\${2}))| #Identify double $ formulas
                                                                                ((\\\[)(.*?)(?<!\\)(\\\]))| #\[
                                                                                ((\\begin\{equation\})(.*?)(?<!\\)(\\end\{equation\}))|
                                                                                ((\\begin\{equation\*\})(.*?)(?<!\\)(\\end\{equation\*\}))| #begin equation with and without *
                                                                                ((\\begin\{align\})(.*?)(?<!\\)(\\end\{align\}))|
                                                                                ((\\begin\{align\*\})(.*?)(?<!\\)(\\end\{align\*\}))| # align and align*
                                                                                ((\\begin\{eqnarray\})(.*?)(?<!\\)(\\end\{eqnarray\}))|
                                                                                ((\\begin\{eqnarray\*\})(.*?)(?<!\\)(\\end\{eqnarray\*\})) #eqnarray and eqnarray*
                                                                        )''',re.DOTALL|re.UNICODE|re.X)


#Label regex
label = re.compile(r'\\label\{(.*?)\}',re.DOTALL|re.UNICODE|re.X)
#List of possible equation delimiters.
delimiters = [r'\(', r'\)', r'\[', r'\]', r'\begin{equation}',r'\begin{equation*}',r'\begin{align}',r'\begin{align*}', r'\end{equation}',r'\end{equation*}',r'\end{align}',r'\end{align*}',r'\begin{eqnarray}', r'\begin{eqnarray*}', r'\end{eqnarray}', r'\end{eqnarray*}',]
#Strings to replace the found formulas.
inlineMathString = "(inlineLaTeXStringNumber%d)"
displayMathString = "(displayLaTeXStringNumber%d)"

#Auxiliar function to be used in seekAndReplaceFormulas to clean the equation from the delimiters.
def cleanDelimiters(equation):
    '''This method clean a string from the possible LaTeX equation delimiters to avoid future conflicts.
            Args:
                    equation(str): The LaTeX equation.
            Returns
                    str: The same equation without the delimiters in the delimiters list. '''
    newEquation = equation
    if(newEquation[0] == '$'):
    #This part is for the case with delimiters $ or $$, since in the equation could be a \$ we did this to avoid cleaning those to.
        newEquation = newEquation[1:len(newEquation)-1]
        #Already cleaned the first and last $ if there is other we pass the cleaner again.
        return cleanDelimiters(newEquation)
    else:
        for delim in delimiters:
            newEquation = newEquation.replace(delim, '')
        return newEquation

#EndOfFunction

#Function to separate the document from the preamble(LaTeX formating) and the information after the document.
#It returns an array of strings with the parts.
def extractContent(completeDocument):
    '''The method takes a string representing a LaTeX document and separates its preamble (the portion before "\\begin{document}"), its content and its epilogue (after \\end{document}) and returns them in a list.
        Args:
            completeDocument(str): The LaTeX document as a string.
        Returns:
            list[str,str,str]: A list of three strings, the first one the portion before \\begin{document}(inclusive), the second one the document, the last one the part after \\end{document}.'''

    preamble = ""
    document = ""
    epilogue = ""

    try:
        preamble = completeDocument[:completeDocument.index(r'\begin{document}')]
        document = completeDocument[completeDocument.index(r'\begin{document}'): completeDocument.index(r'\end{document}')+ len(r'\end{document}')]
        epilogue = completeDocument[completeDocument.index(r'\end{document}')+ len(r'\end{document}'):]
    except ValueError:
        print("\\begin{document} or \end{document} not found.\n")
    return [preamble, document, epilogue]
#EndOfFunction

#HU2 HU8 HU3
#Function to find all the math in the document, copy then in a file and replace then by  markers.
#The input is a strign with the document content, and a string with a name of the file where all the math will be writen.
def seekAndReplaceFormulas(document):
    '''Search for all the math formulas in the document, when one is found first, the function copies the formula in a list; then the function replace the formula for a marker to future uses.
            Args:
                    document(str): the LaTeX content.
            Returns:
                    namedTuple:A named Tuple called documentAndLists where:
                                            replacedDocument is the string with all the formulas replaced by markers.
                                            inline/displayList  is the list with all the inline/display formulas found.'''

    otherDocument = copy.deepcopy(document)

    displayIterator = displayMath.finditer(otherDocument)
    displayIndex = 0
    displayList =[]

    while(True):
        try:
            currentMatch = displayIterator.next()
            otherDocument = otherDocument.replace(currentMatch.group(0), displayMathString%(displayIndex),1)
            currentMatch = cleanDelimiters(currentMatch.group(0))
            displayList.append(currentMatch)
            displayIndex += 1
        except StopIteration:
            break

    inlineIterator = inlineMath.finditer(otherDocument)
    inlineIndex = 0 #Index to keep track of which equation is being replaced.
    inlineList = []

    while(True):
        try:
            currentMatch = inlineIterator.next()
            otherDocument = otherDocument.replace(currentMatch.group(0), inlineMathString%(inlineIndex), 1)
            currentMatch = cleanDelimiters(currentMatch.group(0))
            inlineList.append(currentMatch)
            inlineIndex += 1
        except StopIteration:
            break



    return collections.namedtuple('documentAndLists', 'replacedDocument, inlineList, displayList')(otherDocument, inlineList, displayList)
#EndOfFunction

#HU6
#Insert all the LaTeX formulas in the file formulaFile, already converted, the replacement is done in order of appearance with the marker put before.
def insertConvertedFormulas(htmlString, inlineList, displayList):
    '''Insert all the LaTeX formulas in the lists, already converted, the replacement is done in order of appearance with the marker put before.
            Args:
                    htmlString(str): the html document, product of the conversion from LaTeX to html.
                    inlineList(str): the list with all the inline formulas written in LaTeX.
                    diplayList(str): the list with all the  display formulas written in LaTeX.

            Returns:
                    str: the html document but with all the formulas inserted. '''

    output = ""
    newString = copy.deepcopy(htmlString)
    inlineIndex = 0
    for line in inlineList:
        output = newString.replace(inlineMathString%(inlineIndex), (line) )
        newString = output
        inlineIndex += 1

    output = ""
    displayIndex = 0
    for line in displayList:
        output = newString.replace(displayMathString%(displayIndex),(line) )
        newString = output
        displayIndex += 1

    return newString
#EndOfFunction

#This list contains all the trouble formulas. ¿Is there a way to avoid the global variable?
#TODO: Distinguish between file convertion and unique formula convertion.
troubleFormulas =[]
def reportProblem(strBadFormula):
    '''This function gathers all the formulas the lexer or parser had problems with and then send it to being writed in a file.
            Args:
                    strBadFormula(str): The guilty string. Sended by Parser.'''
    global troubleFormulas
    troubleFormulas.append(strBadFormula)

#EndOfFunction

def generateListOfLabels(listDisplay):
    '''Searchs all the labels and puts them in a list to be look for the references later.
            Args:
                    listDisplay(list(str)): A list with all the display formulas.
            Returns:
                    (list(str)): A list with all the labels found.'''
    labelList =[]
    for formula in listDisplay:
        if(label.search(formula)):
            labelList.append(label.search(formula).group(1))
        else:
            continue

    return labelList

#EndOfFunction

def replaceRefs(stringDocument, listLabels):
    '''Replaces the references in the document for easy to find flags.
            Args:
                    stringDocument(str): The document to look for.
                    listLabels(list(str)): The list with all the labels found.
            Returns:
                    (str):The document with all the references replaced.'''
    replacedDocument = copy.deepcopy(stringDocument)
    output =""
    for label in listLabels:
        output = replacedDocument.replace('\\ref{'+label+'}', '(refTo:'+label+')')
        replacedDocument = output


    return replacedDocument
#EndOfFunction

def insertReferences(stringDocument, listLabels):
    '''Inserts the references changued in the  document.
            Args:
                    stringDocument(str):The document whre the replacement will be done.
                    listLabels(list(str)): The labels in the formulas.
            Returns
                    (str): The document with all the references inserted.'''
    replacedDocument = copy.deepcopy(stringDocument)
    output = ""
    for label in listLabels:
        output = replacedDocument.replace('(refTo:'+label+')','<a tabindex="0" href="#'+label+'" aria-label="'+label+'">r</a>')
        replacedDocument = output


    return replacedDocument
#EndOfFunction



#-*-:coding:utf-8-*-
import re
from . import blindtex
from . import basura
from . import translations

limpiar=basura.basura
reemplazar=translations.translations

numeracion={
	'2': ('0', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'),
	'3': ('0', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X', 'XI', 'XII', 'XIII', 'XIV', 'XV', 'XVI', 'XVII', 'XVIII', 'XIX', 'XX', 'XXI', 'XXII', 'XXIII', 'XXIV', 'XXV', 'XXVI', 'XXVII', 'XXVIII', 'XXIX', 'XXX')
}

#Funciones de transformación de texto básico.
#Transforma el Archivo en un string y quíta lo que está antes de \begin{document} y lo siguiente a \end{document}
def string(nombre_archivo, hijo):
	archivo = open(nombre_archivo, 'r+', encoding='utf-8')
	codigo_sucio = ''
	codigo = ''
	dentro_documento = False
	if hijo == True: dentro_documento = True
	while True:
		lines = archivo.readlines()
		if not lines: break
		for line in lines: codigo_sucio+=line
	patron = re.compile(r'\\input *\{(?P<archivo>[^\{\}#]*)\}')
	ArchivosHijos = patron.findall(codigo_sucio)
	patron=re.compile(r'\.tex$')
	for i in ArchivosHijos:
		if len(patron.findall(i))==0: j=i+'.tex'
		else: j=i
		codigo_sucio = codigo_sucio.replace('\\input{'+i+'}', string(j, True))
	codigo_sucio = codigo_sucio.replace('\r\n', '\n')
	lines=codigo_sucio.split('\n')
	preambulo=''
	macros=list()
	teoremas=list()
	patronmacros=re.compile(r'^\\r?e?newcommand *\{ *\\(?P<nuevocomando>[^#]*) *\} *\{ *\\(?P<comando>[^#]*) *\}$')
	patronmacros2=re.compile(r'^\\providecommand *\{ *\\(?P<nuevocomando>[^#]*) *\} *\{ *\\(?P<comando>[^#]*) *\}$')
	patronteoremas=re.compile(r'^\\newtheorem *\{ *(?P<nuevoteorema>.*) *\}.*\{ *(?P<teorema>.*) *\} *$')
	for line in lines:
		edit_line=line
		if dentro_documento == False:
			if "\\begin{document}" in line: dentro_documento = True
			macros+=patronmacros.findall(line)
			macros+=patronmacros2.findall(line)
			teoremas+=patronteoremas.findall(line)
			if line.strip() == '':
				preambulo += line+'\n'
				continue
			elif line.strip()[0] == "%":
				continue
			elif line.strip()[0] != "%" and '%' in line:
				linea=''
				caracter_especial=False
				for caracter in line:
					if caracter_especial==False and caracter=='%': break
					linea+=caracter
					if caracter_especial==True: caracter_especial=False
					if caracter=='\\': caracter_especial=True
				edit_line=linea
			preambulo+=edit_line+'\n'
		if dentro_documento == True:
			if line.strip() == '':
				codigo += line+'\n'
				continue
			elif line.strip()[0] == "%": continue
			elif line.strip()[0] != "%" and '%' in line:
				linea=''
				caracter_especial=False
				for caracter in line:
					if caracter_especial==False and caracter=='%': break
					linea+=caracter
					if caracter_especial==True: caracter_especial=False
					if caracter=='\\': caracter_especial=True
				edit_line=linea
			if "\\end{document}" in line: break
			codigo+=edit_line+'\n'
	archivo.close()
	fancyfootr=busqueda_avanzada(r'\\fancyfoot *\[ *[Rr] *\] *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(fancyfootr)==1: codigo=fancyfootr[0]+codigo
	fancyfootc=busqueda_avanzada(r'\\fancyfoot *\[ *[Cc] *\] *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(fancyfootc)==1: codigo=fancyfootc[0]+codigo
	fancyfootl=busqueda_avanzada(r'\\fancyfoot *\[ *[Ll] *\] *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(fancyfootl)==1: codigo=fancyfootl[0]+codigo
	fancyheadr=busqueda_avanzada(r'\\fancyhead *\[ *[Rr] *\] *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(fancyheadr)==1: codigo=fancyheadr[0]+codigo
	fancyheadc=busqueda_avanzada(r'\\fancyhead *\[ *[Cc] *\] *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(fancyheadc)==1: codigo=fancyheadc[0]+codigo
	fancyheadl=busqueda_avanzada(r'\\fancyhead *\[ *[Ll] *\] *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(fancyheadl)==1: codigo=fancyheadl[0]+codigo
	date=busqueda_avanzada(r'\\date *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(date)==1: codigo=date[0]+'\n\n'+codigo
	author=busqueda_avanzada(r'\\author *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(author)==1: codigo=author[0]+'\n\n'+codigo
	subtitle=busqueda_avanzada(r'\\subtitle *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(subtitle)==1: codigo=subtitle[0]+'\n\n'+codigo
	title=busqueda_avanzada(r'\\title *\{(?P<texto>.*)\}', preambulo, ('{', '}'))
	if len(title)==1: codigo=title[0]+'\n\n'+codigo
	macros=set(macros)
	macros=list(macros)
	macros.sort(key=len)
	macros.reverse()
	teoremas=set(teoremas)
	teoremas=list(teoremas)
	teoremas.sort(key=len)
	teoremas.reverse()
	for macro in macros:
		macro=(macro[0].replace('\\', '\\\\'), macro[1].replace('\\', '\\\\'))
		codigo=re.sub('\\\\'+macro[0]+'(?P<caracter>[^a-zA-Z])', r'\\'+macro[1]+r'\g<caracter>', codigo)
	for teorema in teoremas:
		teorema=(teorema[0].replace('\\', '\\\\'), teorema[1].replace('\\', '\\\\'))
		codigo=re.sub(r'\\begin\{'+teorema[0]+r'\}(.*\[(?P<teorema>.*)\])*', r'\n'+teorema[1]+r': \g<teorema>\n', codigo)
		codigo=codigo.replace('\\end{'+teorema[0]+'}', '\n')
	codigo=codigo.replace('\r\n', '\n')
	codigo=codigo.replace('\\\\', 'blindtexlineablindtex\\\\blindtexlineablindtex')
	codigo=codigo.replace('\\$', 'blindtexpesoblindtex')
	codigo=codigo.replace('\\(', '$')
	codigo=codigo.replace('\\)', '$')
	codigo=codigo.replace('\\[', '$$')
	codigo=codigo.replace('\\]', '$$')
	codigo=codigo.replace('blindtexlineablindtex\\\\blindtexlineablindtex', '\\\\')
	for i in limpiar['acentos']: codigo=codigo.replace(i[0], i[1])
	codigo=re.sub(r'\\textcolor *\{\w+\} *', r'\\blindtextextcolorblindtex', codigo)
	codigo=sub_avanzado(r'\\blindtextextcolorblindtex *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\blindtextextcolorblindtex *\{.*\}', codigo, ('{', '}')), codigo)
	return codigo

def busqueda_avanzada(patron, texto, delimitador=0):
	# Patrón para encontrar grupos en el parámetro que es un patrón
	# Se quitan los grupos para que en la lista de findall aparezca la expresión completa y no solo el grupo
	patron_quitar_grupo=re.compile(r'\(\?P\<\w*\>(?P<grupocontent>.*)\)', re.S)
	# Se aplica el patrón y queda como string
	string_patron_sin_grupo=re.sub(patron_quitar_grupo, r'\g<grupocontent>', patron)
	# Se genera el patrón usando el string anterior, un patrón sin grupos
	patron_sin_grupo=re.compile(r''+string_patron_sin_grupo, re.S)
	# Lista de encontrados que hay que empezar a reducir
	encontrados_preliminar=patron_sin_grupo.findall(texto)
	# Lista vacía para empezar a añadir los encontrados finales
	encontrados=list()
	# Mientras queden cadenas encontradas por analizar y añadir
	while encontrados_preliminar!=[]:
		# encontrados_preliminar puede contener varios elementos con cadenas, y esos elementos pueden tener varias cadenas
		for preliminar in encontrados_preliminar:
			# Si no se encuentran cadenas retorna lista vacía
			if patron_sin_grupo.findall(preliminar)==[]:
				return []
			else:
				# Patrón con 2 grupos, el primero una cadena cualquiera, el segundo, la última cadena que coincide con el patrón buscado
				# La idea es ir reduciendo la parte restante (el primer grupo) y añadir a la lista encontrados la cadena del segundo grupo
				patron_reductivo=re.compile(r'(?P<restante>.+)(?P<ultimo>'+string_patron_sin_grupo+')', re.S)
				# Si solo queda una cadena en uno de los elementos de encontrados_preliminar por añadir a la lista encontrados
				if len(patron_reductivo.findall(preliminar))==0:
					# Ahora el patrón solo contiene el patrón buscado originalmente pero sin grupos, ya que es la última cadena y no hay parte restante que seguir analizando
					patron_reductivo=re.compile(r''+string_patron_sin_grupo, re.S)
					# Esta es la cadena que posteriormente se añade a la lista encontrados
					reduccion=patron_reductivo.findall(preliminar)[0]
					# Si no hay delimitadores
					if delimitador==0:
						encontrados.append(reduccion)
						# Como ya se añadió a la lista encontrados, se elimina de encontrados_preliminar para que no lo vuelva a analizar
						encontrados_preliminar.remove(preliminar)
					# Si hay delimitadores
					else:
						# Si el caracter anterior al delimitador es \ no pasará al contador
						caracter_especial=False
						# Contador que irá sumando y restando cuando encuentre delimitadores
						cont_delimitadores=0
						# Hace lo mismo que reduccion
						reduccion_delimitadores=""
						for caracter in reduccion:
							# Aún no puede haber \
							if len(reduccion_delimitadores)<=1: caracter_especial=False
							# Si el caracter anterior es \\ se hace falso
							elif reduccion_delimitadores[-1]=='\\': caracter_especial=True
							# Si el caracter especial no está, suma o resta dependiendo del delimitador
							if caracter==delimitador[0] and caracter_especial==False: cont_delimitadores+=1
							if caracter==delimitador[1] and caracter_especial==False: cont_delimitadores-=1
							# Si el delimitador es el último correspondiente, no hay extras al medio y no es un caracter especial, se añade a la cadena y se corta el ciclo
							if cont_delimitadores==0 and caracter_especial==False and delimitador[1]==caracter:
								reduccion_delimitadores+=caracter
								break
							# Se continúa añadiendo caracteres
							reduccion_delimitadores+=caracter
							# Se resetea el verificador de caracter especial
							caracter_especial=False
						# Se añade a encontrados la cadena con los delimitadores correctos
						encontrados.append(reduccion_delimitadores)
						# Como ya se añadió a la lista encontrados, se elimina de encontrados_preliminar para que no lo vuelva a analizar
						encontrados_preliminar.remove(preliminar)
				# En este caso encontrados_preliminar contiene más cadenas para analizar
				else:
					# Se usa el patron_reductivo inicial con 2 grupos
					# Esta variable solo contiene el primer elemento encontrado con el patron_reductivo, que a su vez es una lista de 2 elementos, por los 2 grupos que contiene dicho patrón
					reduccion=patron_reductivo.findall(preliminar)[0]
					# Si no hay delimitadores
					if delimitador==0:
						# Se añade a encontrados_preliminar la parte restante para que en una iteración siguiente sea analizada
						encontrados_preliminar.append(reduccion[0])
						# Se añade a encontrados la cadena perteneciente al grupo 2 del patrón
						encontrados.append(reduccion[1])
						# Se elimina la cadena completa analizada, ya que la primera parte se agregó nuevamente a encontrados_preliminar para ser analizada de nuevo, y la última parte es una cadena ya encontrada y que por lo tanto no requiere ser analizada nuevamente
						encontrados_preliminar.remove(preliminar)
					# Si hay delimitadores
					else:
						# Si el caracter anterior al delimitador es \ no pasará al contador
						caracter_especial=False
						# Contador que irá sumando y restando cuando encuentre delimitadores
						cont_delimitadores=0
						# Hace lo mismo que reduccion
						reduccion_delimitadores=""
						for caracter in reduccion[1]:
							# Aún no puede haber \
							if len(reduccion_delimitadores)<=1: caracter_especial=False
							# Si el caracter anterior es \\ se hace falso
							elif reduccion_delimitadores[-1]=='\\': caracter_especial=True
							# Si el caracter especial no está, suma o resta dependiendo del delimitador
							if caracter==delimitador[0] and caracter_especial==False: cont_delimitadores+=1
							if caracter==delimitador[1] and caracter_especial==False: cont_delimitadores-=1
							# Si el delimitador es el último correspondiente, no hay extras al medio y no es un caracter especial, se añade a la cadena y se corta el ciclo
							if cont_delimitadores==0 and caracter_especial==False and delimitador[1]==caracter:
								reduccion_delimitadores+=caracter
								break
							# Se continúa añadiendo caracteres
							reduccion_delimitadores+=caracter
							# Se resetea el verificador de caracter especial
							caracter_especial=False
						# Se añade a encontrados_preliminar la parte restante para que en una iteración siguiente sea analizada
						encontrados_preliminar.append(reduccion[0])
						# Se añade a encontrados la cadena con los delimitadores correctos
						encontrados.append(reduccion_delimitadores)
						# Se elimina la cadena completa analizada, ya que la primera parte se agregó nuevamente a encontrados_preliminar para ser analizada de nuevo, y la última parte es una cadena ya encontrada y que por lo tanto no requiere ser analizada nuevamente
						encontrados_preliminar.remove(preliminar)
	# En estas 4 líneas se borran elementos duplicados y se ordena de mayor longitud a menor para que al traducir no hayan conflictos
	encontrados=set(encontrados)
	encontrados=list(encontrados)
	encontrados.sort(key=len)
	encontrados.reverse()
	encontrados_final=list()
	# Ahora se usa el patron_con_grupo para obtener lo que realmente se buscaba
	patron_con_grupo=re.compile(patron, re.S)
	for encontrado in encontrados:
		if string_patron_sin_grupo==patron: encontrados_final.append(encontrado)
		else: encontrados_final.append(patron_con_grupo.findall(encontrado)[0])
	return encontrados_final

def sub_avanzado(patron, patron_sub, lista, texto):
	patron_subear=re.compile(patron, re.S)
	for i in range(len(lista)):
		texto=texto.replace(lista[i], re.sub(patron_subear, patron_sub, lista[i]))
	return texto

def buscar_formulas(codigo, valor):
	inlineMath = re.compile(r''' #Exclude escaped symbols
		\$[^\$]*?\$| #Single $ formulas
		\\\(.*?\\\) #\(
	''', re.DOTALL|re.UNICODE|re.X|re.S)
	displayMath = re.compile(r''' #Exclude escaped symbols
		\${2}.*?\${2}| #Identify double $ formulas
		\\\[.*?\\\]| #\[
		\\begin *\{ *equation *\}.*?\\end *\{ *equation *\}|
		\\begin *\{equation *\* *\}.*?\\end *\{ *equation *\* *\}| #begin equation with and without *
		\\begin *\{ *align *\}.*?\\end *\{ *align *\}|
		\\begin *\{ *align *\* *\}.*?\\end *\{ *align *\* *\}| # align and align*
		\\begin *\{ *eqnarray *\}.*?\\end *\{ *eqnarray *\}|
		\\begin *\{ *eqnarray *\* *\}.*?\\end *\{ *eqnarray *\* *\}| #eqnarray and eqnarray*
		\\begin *\{ *multline *\}.*?\\end *\{ *multline *\}|
		\\begin *\{ *multline *\* *\}.*?\\end *\{ *multline *\* *\} #multline and multline*
	''',re.DOTALL|re.UNICODE|re.X|re.S)
	#Crea una lista "forms" con los resultados de las dos busquedas anteriores
	if valor==1: forms=inlineMath.findall(codigo)
	else: forms=displayMath.findall(codigo)
	#Se ajusta el formato de la lista de salida
	forms=set(forms)
	forms=list(forms)
	forms.sort(key=len)
	forms.reverse()
	return forms

def buscar_codigos(codigo):
	patron = re.compile(r''' #Exclude escaped symbols
		\\begin *\{ *verbatim *\}.*?\\end *\{ *verbatim *\}
	''',re.DOTALL|re.UNICODE|re.X|re.S)
	codes= patron.findall(codigo)
	codes=list(codes)
	codes.sort(key=len)
	codes.reverse()
	return codes

def reemplazar_formulas(codigo, formulas, tipo, valor):
	if valor=='r':
		cont=1
		for formula in formulas:
			codigo=codigo.replace(formula, 'blindtex'+tipo+str(cont)+'blindtex'+tipo)
			cont+=1
	elif valor=='w':
		cont=1
		for formula in formulas:
			codigo=codigo.replace('blindtex'+tipo+str(cont)+'blindtex'+tipo, formula)
			cont+=1
	return codigo

#Traduce todas las fórmulas
def traducir_formulas(formulas):
	for i in formulas:
		patron= re.compile(r"\$(?P<formula>[^\$]*?)\$", re.S)
		patron2=re.compile(r"\\begin *\{ *equation *\** *\}[^\n]*\s*(?P<formula>.*?)\s*\\end *\{ *equation *\** *\}", re.S)
		patron3=re.compile(r"\\begin *\{ *align *\** *\}[^\n]*\s*(?P<formula>.*?)\s*\\end *\{ *align *\** *\}", re.S)
		patron4=re.compile(r"\\begin *\{ *eqnarray *\** *\}[^\n]*\s*(?P<formula>.*?)\s*\\end *\{ *eqnarray *\** *\}", re.S)
		patron5=re.compile(r"\\begin *\{ *multline *\** *\}[^\n]*\s*(?P<formula>.*?)\s*\\end *\{ *multline *\** *\}", re.S)
		patron6=re.compile(r"\${2}\s*(?P<formula>.*?)\s*\${2}", re.S)
		formula=patron.findall(i)+patron2.findall(i)+patron3.findall(i)+patron4.findall(i)+patron5.findall(i)+patron6.findall(i)
		formula.sort(key=len)
		formula.reverse()
		#utiliza blindtex para traducir los patrones encontrados anteriormente, reemplazandolos en el archivo original
		try:
			j=formula[0]
			for expresion in reemplazar['basura_latex']:
				j=j.replace(expresion+" ", ' ')
				j=j.replace(expresion+"\n", ' ')
				j=j.replace(expresion+"-", '-')
				j=j.replace(expresion+"+", '+')
				j=j.replace(expresion+"^", '^')
				j=j.replace(expresion+"_", '_')
				j=j.replace(expresion+"\\", '\\')
				j=j.replace(expresion+"|", '|')
			for simbolo in reemplazar['simbolos']: j=j.replace(simbolo[0], simbolo[1])
			for expresion in reemplazar['reemplazos_latex']: j=j.replace(expresion[0], expresion[1])
			j = re.sub(r'\\[a-z]space *\{[^\{\}]*\}', r'', j)
			j = re.sub(r'\\not *\\in', r'\\notin', j)
			j = re.sub(r'\\math[a-z]+ *\{(?P<texto>[^\{\}]+)\}', r'\g<texto>', j)
			j = re.sub(r'\\math[a-z]{2,3} (?P<texto>[A-Za-z0-9])', r'\g<texto>', j)
			j=re.sub(r'\\textnormal *\{', r'\\text{', j)
			j=re.sub(r'\\text[a-z]+ *\{', r'\\text{', j)
			j = re.sub(r'\\bm *\{(?P<texto>[^\{\}]*)\}', r'\g<texto>', j)
			j = re.sub(r'\\begin *\{ *aligned *\} *\[[a-z]\]', '', j)
			j = re.sub(r'\\begin *\{ *aligned *\}', '', j)
			j = re.sub(r'\\end *\{ *aligned *\}', '', j)
			j = re.sub(r'\\begin *\{ *cases *\}', '', j)
			j = re.sub(r'\\end *\{ *cases *\}', '', j)
			for _ in range(5):
				j=sub_avanzado(r'\\mathop *\{(?P<dentro>.*)\}', r'\g<dentro>', busqueda_avanzada(r'\\mathop *\{.*\}', j, ('{', '}')), j)
				j=sub_avanzado(r'\\boxed *\{(?P<dentro>.*)\}', r'\g<dentro>', busqueda_avanzada(r'\\boxed *\{.*\}', j, ('{', '}')), j)
				j=sub_avanzado(r'\\operatorname *\{(?P<dentro>.*)\}', r'\g<dentro>', busqueda_avanzada(r'\\operatorname *\{.*\}', j, ('{', '}')), j)
				j=sub_avanzado(r'\\intertext *\{(?P<dentro>.*)\}', r'blindtexlineablindtex \\text{\g<dentro>} blindtexlineablindtex', busqueda_avanzada(r'\\intertext *\{.*\}', j, ('{', '}')), j)
				j=sub_avanzado(r'\\vec *\{(?P<vector>\w)\}', r'\\vec \g<vector>', busqueda_avanzada(r'\\vec *\{\w\}', j, ('{', '}')), j)
				j=sub_avanzado(r'\\vec *\{(?P<vector>.*)\}', r' vecblindtexparentesisblindtex\g<vector>blindtexcierraparentesisblindtex', busqueda_avanzada(r'\\vec *\{.*\}', j, ('{', '}')), j)
				j=sub_avanzado(r'\\overline *\{(?P<overline>.*)\}', r'bar blindtexparentesisblindtex\g<overline>blindtexcierraparentesisblindtex', busqueda_avanzada(r'\\overline *\{.*\}', j, ('{', '}')), j)
			j=re.sub(r'\\.?frac(?P<numerador>[^\{\}])(?P<denominador>.)', r'\\frac{\g<numerador>}{\g<denominador>}', j)
			j=re.sub(r'(?P<in>\\in)(?P<fin>[^a-z])', r'blindtexperteneceblindtex\g<fin>', j)
			patronmatrices=re.compile(r"\\begin *\{ *.matrix *\}\s*.+?\s*\\end *\{ *.matrix *\}", re.S)
			patronmatrices2=re.compile(r"\\begin *\{ *array *\} *.*? *\{ *[\| a-z]* *\}\s*.+?\s*\\end *\{ *array *\}", re.S)
			lista_matrices=patronmatrices.findall(j)+patronmatrices2.findall(j)
			lista_matrices=set(lista_matrices)
			lista_matrices=list(lista_matrices)
			lista_matrices.sort(key=len)
			lista_matrices.reverse()
			for matriz in lista_matrices:
				matriz_arreglada=matriz
				if ('=' or '\\neq' or '\\ne ' or '<' or '>' or '\\leq' or '\\geq' or '\\le ' or '\\ge ') in matriz:
					matriz_arreglada=re.sub(r"\\begin\{array\} *.*? *\{ *[\| a-z]* *\}\s*", r'blindtexlineablindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r"\s*\\end\{array\}", r'blindtexlineablindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r'\s*blindtexlineablindtex\s*', r'blindtexlineablindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r' *& *', r'', matriz_arreglada)
				else:
					matriz_arreglada=re.sub(r"\\begin *\{ *.matrix *\}\s*", r'blindtexparentesisblindtex\nblindtexparentesisblindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r"\\begin *\{ *array *\} *.*? *\{ *[\| a-z]* *\}\s*", r'blindtexparentesisblindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r"\s*\\end *\{ *.matrix *\}", r'blindtexcierraparentesisblindtexblindtexcierraparentesisblindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r"\s*blindtexlineablindtex\s*\\end *\{ *array *\}", r'blindtexcierraparentesisblindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r"\s*\\end *\{ *array *\}", r'blindtexcierraparentesisblindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r'\s*blindtexlineablindtex\s*', r'blindtexcierraparentesisblindtex,blindtexlineablindtexblindtexparentesisblindtex', matriz_arreglada)
					matriz_arreglada=re.sub(r' *& *', r',', matriz_arreglada)
				j=j.replace(matriz, matriz_arreglada)
			j=re.sub(r' {2,10}', r' ', j)
			j=j.replace('\\ ', ' ')
			if j.strip() == "": continue
			x=blindtex.tex2all.read_equation(j)
			x=x.replace('b l i n d t e x l i n e a b l i n d t e x', '\n')
			patrontextos= re.compile("ç(?P<texto>[^ç]+)endtext", re.MULTILINE)
			match_possition = 0
			for match in patrontextos.finditer(x):
				texto = x[match.start()+match_possition+1:match.end()+match_possition-7]
				if texto.strip() == "": continue
				texto = texto.replace('&', 'blindtexampersandblindtex')
				texto = texto.replace(' ', 'blindtexespacioblindtex')
				x = x[:match.start()+match_possition] + texto + x[match.end()+match_possition:]
				diff_match = match.end() - match.start()
				if len(texto) < diff_match:
					match_possition -= diff_match - len(texto)
				elif len(texto) > diff_match:
					match_possition += len(texto) - diff_match
			x=x.replace(' ', '')
			x=x.replace(':', ': ')
			x=x.replace('çendtext', ' ')
			x=x.replace('linebreak', '')
			x=x.replace('blindtexparentesisblindtex', '(')
			x=x.replace('blindtexcierraparentesisblindtex', ')')
			x=x.replace('&', '')
			x=x.replace('blindtexampersandblindtex', '&')
			x=x.replace('blindtexperteneceblindtex', '∈')
			for simbolo in reemplazar['simbolos']: x=x.replace(simbolo[1], simbolo[2])
			for expresion in reemplazar['reemplazos_traduccion']: x=x.replace(expresion[0], expresion[1])
			x=re.sub(r'\^\((?P<exponente>.)\)', r'^\g<exponente>', x)
			x=re.sub(r'_\((?P<subindice>.)\)', r'_\g<subindice>', x)
			x=re.sub(r'√\((?P<raiz>.)\)', r'√\g<raiz>', x)
			formulas[formulas.index(i)]=x
		except:
			pass
	return formulas

#Se deshace de los graficos encontrados en el codigo LaTeX
def graficos(codigo):
	codigo = re.sub(r"\\includegraphics *\[ *(?P<tiende>.*) *\] *\{ *(?P<nombre_grafico>.*) *\}", r"Gráfico \g<nombre_grafico>", codigo)
	return codigo

#Extrae los titulos del código LaTeX
def titulos(codigo):
	codigo = re.sub(r"\\frametitle\{(?P<titulo>.*)\}", r"\g<titulo>\n", codigo)
	codigo = re.sub(r"\\framesubtitle\{(?P<titulo>.*)\}", r"\g<titulo>\n", codigo)
	codigo = re.sub(r"\\begin\{block\}\{(?P<titulo_bloque>.*)\}", r"\g<titulo_bloque>\n", codigo)
	codigo = re.sub(r"\\begin\{frame\}.*\{*(?P<titulo_cuadro>.*)\}*", r"\g<titulo_cuadro>\n", codigo)
	return codigo

def enumeracion(codigo):
	codigo = re.sub(r'\\item\s*\[(?P<texto>[^\[]*)\]\s*', r'\g<texto>\t', codigo)
	cont = list()
	codigo2=''
	lines_codigo=codigo.split('\n')
	for line in lines_codigo:
		if "\\begin{enumerate}" in line:
			cont.append(1)
			itemize = 0
		if "\\end{enumerate}" in line: cont.remove(cont[-1])
		# Verificando que solo se reemplace item de itemize/description y no de enumerate
		if cont != list() and "\\begin{itemize}" in line: itemize+=1
		if cont != list() and "\\end{itemize}" in line: itemize=itemize-1
		if cont != list() and "\\begin{description}" in line: itemize+=1
		if cont != list() and "\\end{description}" in line: itemize=itemize-1
		if cont != list() and "\\item" in line and itemize==0:
			if len(cont)==1: numero=str(cont[-1])
			if len(cont)==2: numero=str(numeracion['2'][cont[-1]])
			if len(cont)==3: numero=str(numeracion['3'][cont[-1]])
			line = re.sub(r'\\item', r''+numero+'.\tblindtexenumeracion', line)
			cont[-1] += 1
		codigo2+=line+'\n'
	codigo=codigo.replace(codigo, codigo2)
	codigo = re.sub(r'blindtexenumeracion\s*', r'', codigo)
	codigo = re.sub(r'\\item\s*\[(?P<texto>[^\[]*)\]\s*', r'\g<texto>\t', codigo)
	codigo = re.sub(r'\\item\s*', r'•\t', codigo)
	for i in limpiar['enumeracion']: codigo=codigo.replace(i[0], i[1])
	return codigo

def split_codigo(codigo):
	codigo_splited=codigo.split('\n')
	codigo2=''
	for i in codigo_splited: codigo2+=i.strip()+'\n'
	return codigo2

#Elimina código de formato y otros en el archivo, para que estos no se muestren en el texto final
def limpiar_basura(codigo):
	codigo=codigo.replace('blindtexpesoblindtex', '$')
	codigo = re.sub(r'\{ *\\large *(?P<texto>.*)\}', r'\g<texto>', codigo)
	codigo = re.sub(r'\{ *\\LARGE *(?P<texto>.*)\}', r'\g<texto>', codigo)
	codigo = re.sub(r'\{ *\\Large *(?P<texto>.*)\}', r'\g<texto>', codigo)
	codigo = re.sub(r'\\begin *\{ *alertblock *\*? *\} *\{(?P<texto>.*?)\}', r'\g<texto>', codigo)
	codigo = re.sub(r'\\end *\{ *alertblock *\*? *\}', r'', codigo)
	codigo = re.sub(r'\\begin *\{ *textblock *\*? *\}.*', r'', codigo)
	codigo = re.sub(r'\\end *\{ *textblock *\*? *\}', r'', codigo)
	codigo = re.sub(r'\\begin *\{ *minipage *\}.*', r'', codigo)
	codigo = re.sub(r'\\end *\{ *minipage *\}', r'', codigo)
	codigo = re.sub(r'\\setcounter *\{ *enumi *\} *\{ *\d* *\}', r'', codigo)
	codigo = re.sub(r'\\thispagestyle *\{.*\}', r'', codigo)
	codigo = re.sub(r'\\begin *\{ *multicols *\} *\{ *\d* *\}', r'', codigo)
	codigo = re.sub(r'\\.space *\** *\{[^\{\}]*?\}', r'', codigo)
	for _ in range(3):
		codigo=sub_avanzado(r'\\only *< *[\d-]* *> *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\only *< *[\d-]* *> *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\scshape *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\scshape *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\small *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\small *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\small *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\small *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\scriptsize *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\scriptsize *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\[a-z]box *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\[a-z]box *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\[a-z]box *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\[a-z]box *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\text\w* *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\text\w* *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\text\w* *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\text\w* *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\emph *(?P<texto>[^\{\}]+)\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\emph *[^\{\}]+\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\emph *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\emph *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\emph *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\emph *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\em *(?P<texto>[^\{\}]+)\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\em *[^\{\}]+\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\em *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\em *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\em *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\em *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\it *(?P<texto>[^\{\}]+)\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\it *[^\{\}]+\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\it *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\it *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\it *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\it *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\bf *(?P<texto>[^\{\}]+)\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\bf *[^\{\}]+\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\bf *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\bf *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\bf *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\bf *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\large *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\large *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\large *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\large *\{.*\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\{ *\\hfill *\{(?P<texto>.*)\} *\}', r'\g<texto>', busqueda_avanzada(r'\{ *\\hfill *\{.*\} *\}', codigo, ('{', '}')), codigo)
		codigo=sub_avanzado(r'\\hfill *\{(?P<texto>.*)\}', r'\g<texto>', busqueda_avanzada(r'\\hfill *\{.*\}', codigo, ('{', '}')), codigo)
	codigo=split_codigo(codigo)
	codigo=re.sub(r'\n{2,}', r'blindtexlineasblindtex', codigo)
	codigo = re.sub(r'(?P<caracter1>[^\}\\])\n(?P<caracter2>[^\\])', r'\g<caracter1> \g<caracter2>', codigo)
	codigo=re.sub(r'(?P<ecuacion>blindtexecuacion\d+blindtexecuacion)', r'\n\g<ecuacion>\n', codigo)
	codigo=codigo.replace('blindtexlineasblindtex', '\n\n')
	codigo=re.sub(r'\\hypertarget *\{[^\{\}]*\} *\{\s*\n*\s*(?P<texto>\\[sub]*section *\{.*\} *\\label *\{.*\}) *\}', r'\g<texto>', codigo)
	for i in limpiar['delimitadores']: codigo=codigo.replace(i[0], i[1])
	for i in limpiar['simples']: codigo=codigo.replace(i[0], i[1])
	codigo=codigo.strip()
	codigo=split_codigo(codigo)
	return codigo

def traducido(codigo, nombre_archivo):
	final=open(nombre_archivo, encoding='utf-8', mode='w')
	final.write(codigo)
	final.close()
